import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;
/** 
 * SnakeOil (c) 2015 - 2019
 * https://bitbucket.org/MysticLord
 */
public final class DataHandler
{
    public static final Integer[] ARRAY_UNSIGNED_BYTE =  genIntArray(0, 255, new Integer[256]);
    public static final Integer[] ARRAY_SIGNED_BYTE = genIntArray(-128, 127, new Integer[256]);
    public static final Integer[] ARRAY_UNSIGNED_AND_SIGNED_BYTE = genIntArray(-128, 255, new Integer[384]);
    public static final Integer[] ARRAY_UNSIGNED_HALF_BYTE = genIntArray(0, 127, new Integer[128]);
    public static final Integer[] ARRAY_UNSIGNED_QUARTER_BYTE = genIntArray(0, 63, new Integer[64]);

    public static final String[] ARRAY_STRING_NIBBLE = genStrIntArray(genIntArray(0, 15, new Integer[16]));
    public static final String[] ARRAY_STRING_UNSIGNED_QUARTER_BYTE = genStrIntArray(ARRAY_UNSIGNED_QUARTER_BYTE);
    public static final String[] ARRAY_STRING_UNSIGNED_HALF_BYTE = genStrIntArray(genIntArray(0, 127, new Integer[128]));
    public static final String[] ARRAY_STRING_UNSIGNED_BYTE = genStrIntArray(ARRAY_UNSIGNED_BYTE);

    public static final String[] ARRAY_HEX_STRING_UNSIGNED_BYTE = genHexStrIntArray(ARRAY_UNSIGNED_BYTE);
    public static final String[] ARRAY_HEX_STRING_UNSIGNED_QUARTER_BYTE = genHexStrIntArray(ARRAY_UNSIGNED_QUARTER_BYTE);

    public static final String[] ARRAY_STRING_ITEMS = genStringArray("itemNames.txt", 256);
    public static final String[] ARRAY_STRING_ABILITIES = genStringArray("abilityNames.txt", 512);
    public static final String[] ARRAY_STRING_ABILITIES_LOW_BYTE = genStringArray("abilityNames.txt", 256);
    public static final String[] ARRAY_STRING_ACTIONS = genStringArray("actionNames.txt", 5);
    public static final String[] ARRAY_STRING_MAGIC = genStringArray("abilityNames.txt", 256, 352);
    public static final String[] ARRAY_STRING_GUN_TECHS = genStringArray("abilityNames.txt", 0x40, 0x50);
    public static final String[] ARRAY_STRING_ENEMIES = genStringArray("enemyNames.txt",256);
    public static final String[] ARRAY_STRING_CHARACTERS = genStringArray("characterNames.txt",40);

    public static LinkedList<FixedSizeDataTable> dataTables;
    public static LinkedList<PS1Path> discFiles;
    private DataHandler(){}

    /**
     * Generate sequence of integers from floor to ceiling, inclusive both ends.
     * @param int floor
     * @param int ceiling
     * @param Integer[] target
     * @return Integer[] target
     */
    public static Integer[] genIntArray (int floor, int ceiling, Integer[] target)
    {
        if (floor < ceiling)
        {
            for (int i = 0; floor <= ceiling; floor++, i++)
            {
                target[i] = floor;
            }
            return target;
        }
        else
        {
            System.out.println("Not a valid genArray(...) call.");
            return null;
        }
    }

    /**
     * Generate sequence of integers as Strings, using an array of Integers.
     * @param   Integer[] inputArr
     * @return  String[] outputArr
     */
    public static String[] genStrIntArray (Integer[] inputArr)
    {
        String[] outputArr = new String[inputArr.length];
        for (int i = 0; i < inputArr.length; i++)
        {
            outputArr[i] = Integer.toString(inputArr[i]);
        }
        return outputArr;
    }

    /**
     * Generate sequence of integers as Hex Strings, using an array of Integers.
     * @param   Integer[] inputArr
     * @return  String[] outputArr
     */
    public static String[] genHexStrIntArray(Integer[] inputArr)
    {
        String[] outputArr = new String[inputArr.length];
        for (int i = 0; i < inputArr.length; i++)
        {
            if (i > 16)
            {
                outputArr[i] = "0x" + Integer.toHexString(inputArr[i]);
            }
            else
            {
                outputArr[i] = "0x0" + Integer.toHexString(inputArr[i]);
            }
        }
        return outputArr;
    }

    /**
     * Make a String[] of int size, from lines of given text file in names.zip.
     * @param String fileName
     * @param int size
     * @return String[] returnValue
     */
    public static String[] genStringArray(String fileName, int size)
    {
        //maybe switch to something other than a zip file, since java has trouble with them?
        //why not put them in ./config/names?
        //honestly kinda stupid, thanks java (and thanks linux)
        ZipFile zipFile = null;
        //flush and close InputStream, InputStreamReader, BufferedReader in finally block below, instantiate them above as null objects.
        try
        {
            zipFile = new ZipFile(new File("names.zip"));
            ZipEntry fileEntry = zipFile.getEntry(fileName);
            InputStream iStream = zipFile.getInputStream(fileEntry);
            InputStreamReader iSReader = new InputStreamReader(iStream, "UTF-8");
            BufferedReader bReader = new BufferedReader(iSReader);
            String[] returnValue = new String[size];
            String tempString = "";
            for (int i = 0; tempString != null && i < size; i++)
            {
                tempString = bReader.readLine();
                returnValue[i] = tempString;
            }
            return returnValue;
        }
        catch(Exception e)
        {
            //need some way to report an error via gui object if file is missing...
            System.out.println("Exception thrown in genStringArray(...) in DataHandler.java");
            e.printStackTrace();
            return null;
        }
        finally
        {
            try
            {
                if (zipFile != null)
                {
                    zipFile.close();
                }
            }
            catch (Exception e)
            {
                ;//do nothing
            }
        }
    }

    /**
     * Make a String[] of size ceiling - floor, from lines of given text file in names.zip.
     * @param String fileName
     * @param int size
     * @return String[] returnValue
     */
    public static String[] genStringArray(String fileName, int floor, int ceiling)
    {
        ZipFile zipFile = null;
        try
        {
            zipFile = new ZipFile(new File("names.zip"));
            ZipEntry fileEntry = zipFile.getEntry(fileName);
            InputStream iStream = zipFile.getInputStream(fileEntry);
            InputStreamReader iSReader = new InputStreamReader(iStream, "UTF-8");
            BufferedReader bReader = new BufferedReader(iSReader);
            String[] returnValue = new String[ceiling - floor];
            String tempString = "";
            for (int i = 0; tempString != null && i < ceiling; i++)
            {
                tempString = bReader.readLine();
                String testStr1 = "0x" + tempString.substring(0,3);
                int testValue = Integer.decode(testStr1);
                if (testValue >= floor)//maybe move this test to for loop in future
                {
                    returnValue[i - floor] = tempString;
                }
            }
            return returnValue;
        }
        catch(Exception e)
        {
            System.out.println("Exception thrown in genStringArray(...) in DataHandler.java");
            e.printStackTrace();
            return null;
        }
        finally
        {
            try
            {
                if (zipFile != null)
                {
                    zipFile.close();
                }
            }
            catch (Exception e)
            {
                ;//do nothing
            }
        }
    }

    /**
     * Method to load data from a patch into various data structures.
     * @param   String Filename and path of the selected patch.
     * @return  boolean result
     */
    public static boolean readFromPatch(String selectedPatch)
    {
        FileInputStream fis = null;
        ZipInputStream zis = null;
        boolean result = false;
        try
        {
            //Temporary data.csv data storage
            LinkedList<String[]> tempList = new LinkedList<String[]>();
            String dataPath = "./data.csv";
            BufferedReader br = new BufferedReader(new FileReader(dataPath));
            for (String line = br.readLine(); line != null; line = br.readLine()){
                tempList.add(line.split(","));
            }
            br.close();

            dataTables = new LinkedList<FixedSizeDataTable>();//initialize dataTables

            Iterator<String[]> iterator = tempList.iterator();
            while (iterator.hasNext()){//Put data.csv information in dataTables
                String[] strArr = iterator.next();
                if (strArr[2].equals("absolute")){
                    dataTables.add(new AbsoluteAddressDataTable(strArr[0], 
                            strArr[1], Integer.parseInt(strArr[3]),
                            Integer.parseInt(strArr[4]), Integer.parseInt(strArr[5]), 
                            Long.parseLong(strArr[6])));
                }
                else if (strArr[2].equals("pointered")){
                    dataTables.add(new PointerAddressedDataTable(strArr[0], 
                            strArr[1], Integer.parseInt(strArr[3]),
                            Integer.parseInt(strArr[4]), Integer.parseInt(strArr[5]),
                            Integer.parseInt(strArr[6]), 
                            Boolean.parseBoolean(strArr[7]),
                            Long.parseLong(strArr[8])));
                }//New entries for variable length data tables, images, music, and video later
            }

            fis = new FileInputStream(new File(selectedPatch));
            zis = new ZipInputStream(fis);
            ZipEntry zipEntry = null;
            while ((zipEntry = zis.getNextEntry()) != null)
            {//iterate through ZipInputStream...
                for (FixedSizeDataTable dataTable : dataTables)
                {//iterate through LinkedList<FixedSizeDataTable>...
                    String tempTableName = dataTable.getTableName();
                    if (zipEntry.getName().equals(tempTableName))
                    {//if dataTable name matches file in ZipInputStream...
                        int entrySize = dataTable.getEntrySize();
                        dataTable.data = new HashMap<String,Integer[]>();
                        for (int i = 0; i < dataTable.getNumOfEntries(); i++)
                        {
                            int expectedNumberOfBytes = entrySize;
                            byte[] byteBuffer = new byte[entrySize];
                            for (int totalBytesRead = 0, offset = 0; totalBytesRead < expectedNumberOfBytes;)
                            {
                                int numberOfBytesRemaining = expectedNumberOfBytes - totalBytesRead;
                                int bytesRead = zis.read(byteBuffer, offset, numberOfBytesRemaining);
                                if (bytesRead == -1)
                                {
                                    throw new IOException("The expected number of bytes was not " + 
                                        "found for" + zipEntry.getName());
                                }
                                totalBytesRead += bytesRead;
                                offset += bytesRead;
                            }
                            Integer[] intBuffer = new Integer[entrySize];

                            for (int n = 0; n < entrySize; n++)
                            {
                                intBuffer[n] = (byteBuffer[n] & 0xff);
                            }
                            //abstract the code below by adding the name of the string array to each data.csv entry
                            if (tempTableName.equals("diffTables"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i], intBuffer);
                            }
                            if (tempTableName.equals("items1"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_ITEMS[i], intBuffer);
                            }
                            else if (tempTableName.equals("mysticAbsorbs"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_ENEMIES[i], intBuffer);
                            }
                            else if (tempTableName.equals("monsterForms1"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i], intBuffer);
                            }
                            else if (tempTableName.equals("monsterForms2"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i], intBuffer);
                            }
                            else if (tempTableName.equals("skills1"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_ABILITIES[i], intBuffer);
                            }
                            else if (tempTableName.equals("actionGrowths"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_ACTIONS[i], intBuffer);
                            }
                            else if (tempTableName.equals("magicLearning"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_MAGIC[i], intBuffer);
                            }
                            else if (tempTableName.equals("gunLearning"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_GUN_TECHS[i], intBuffer);
                            }
                            else if (tempTableName.equals("mecSelfDev"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_UNSIGNED_QUARTER_BYTE[i], intBuffer);
                            }
                            else if (tempTableName.equals("initialCharacter1"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_CHARACTERS[i], intBuffer);
                            }
                            else if (tempTableName.equals("initialCharacter2"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_CHARACTERS[i], intBuffer);
                            }
                            else if (tempTableName.equals("initialCharacter3"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_CHARACTERS[i], intBuffer);
                            }
                            else if (tempTableName.equals("initialCharacter4"))
                            {
                                dataTable.data.put(DataHandler.ARRAY_STRING_CHARACTERS[i], intBuffer);
                            }
                        }
                        break;
                    }

                }
            }
            zis.close();
            fis.close();
            result = true;
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (zis != null)
            {
                try
                {
                    zis.close();
                }
                catch (Exception e){};
            }
            if (fis != null)
            {
                try
                {
                    fis.close();
                }
                catch (Exception e){};
            }
            return result;
        }
    }

    /**
     * Method to write data from various data structures to a patch.
     * @param   String Filename and path of the selected patch.
     * @return  boolean result
     */
    public static boolean writeToPatch(String selectedPatch)
    {
        ZipOutputStream zos = null;
        boolean result = false;
        try
        {
            zos = new ZipOutputStream(new FileOutputStream(new File(selectedPatch)));
            for (FixedSizeDataTable dataTable : dataTables)
            {//iterate through LinkedList<FixedSizeDataTable>...
                int entrySize = dataTable.getEntrySize();
                String name = dataTable.getTableName();
                ZipEntry currEntry = new ZipEntry(name);
                zos.putNextEntry(currEntry);
                for(int i = 0; i < dataTable.getNumOfEntries(); i++)
                {//For every entry in given data table...
                    Integer[] intBuffer = new Integer[entrySize];
                    byte[] byteBuffer = new byte[entrySize];
                    //abstract code below by adding the name of the string array to each data.csv entry
                    if (name.equals("diffTables"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i]);
                    }
                    if (name.equals("items1"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_ITEMS[i]);
                    }
                    else if (name.equals("mysticAbsorbs"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_ENEMIES[i]);
                    }
                    else if (name.equals("monsterForms1"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i]);
                    }
                    else if (name.equals("monsterForms2"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i]);
                    }
                    else if (name.equals("skills1"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_ABILITIES[i]);
                    }
                    else if (name.equals("actionGrowths"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_ACTIONS[i]);
                    }
                    else if (name.equals("magicLearning"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_MAGIC[i]);
                    }
                    else if (name.equals("gunLearning"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_GUN_TECHS[i]);
                    }
                    else if (name.equals("mecSelfDev"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_UNSIGNED_QUARTER_BYTE[i]);
                    }
                    else if (name.equals("initialCharacter1"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_CHARACTERS[i]);
                    }
                    else if (name.equals("initialCharacter2"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_CHARACTERS[i]);
                    }
                    else if (name.equals("initialCharacter3"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_CHARACTERS[i]);
                    }
                    else if (name.equals("initialCharacter4"))
                    {
                        intBuffer = dataTable.data.get(DataHandler.ARRAY_STRING_CHARACTERS[i]);
                    }
                    for (int n = 0; n < entrySize; n++)
                    {
                        byteBuffer[n] = intBuffer[n].byteValue();
                    }
                    zos.write(byteBuffer, 0, entrySize);
                }
                zos.closeEntry();
            }
            result = true;
        }
        catch (Exception e)
        {
            System.out.println("Exception thrown in method writeToPatch(...)");
            System.out.println(e);
            e.printStackTrace();
        }
        finally
        {
            try
            {
                if (zos != null)
                {
                    zos.flush();
                    zos.close();
                }
            }
            catch (Exception e){;}
            return result;
        }
    }

    /**
     * Reads directory structure of disc image and initializes LinkedList<PS1Paths> 
     * discFiles.  Use prior to iterating through discFiles.
     * @param   RandomAccessFile disc
     * @return  boolean result
     */
    private static boolean readDirectories(RandomAccessFile disc) throws Exception
    {
        boolean result = false;
        try
        {
            long primaryDirFirstEntry = 51768L;
            disc.seek(primaryDirFirstEntry);//go to first record
            int recordLength = disc.readUnsignedByte();

            disc.skipBytes(recordLength + 9);//skip to file/dir size of second record (curr dir)

            //32 bit (4 byte) unsigned int, read data in via bit shift and logical "or"
            int primaryDirLength = (disc.readByte() & 0xff);
            primaryDirLength = (((disc.readByte() << 8) & 0xffff) | primaryDirLength);
            primaryDirLength = (((disc.readByte() << 16) & 0xffffff) | primaryDirLength);
            primaryDirLength = (((disc.readByte() << 24) & 0xffffffff) | primaryDirLength);

            LinkedList<PS1Path> nonPrimaryDirs = new LinkedList<PS1Path>();
            discFiles = new LinkedList<PS1Path>();

            byte[] byteBuffer = new byte[primaryDirLength];
            disc.seek(primaryDirFirstEntry);
            if (primaryDirLength <= 2048)
            {
                disc.readFully(byteBuffer);
            }
            else if (primaryDirLength > 2048)
            {
                int numOfSectors = primaryDirLength / 2048;
                for (int i = 0; i < numOfSectors; i++)
                {
                    disc.read(byteBuffer, i * 2048, 2048);
                    disc.seek(primaryDirFirstEntry + (i * 2352));
                }
            }

            int counterPrimary = 0;
            while (counterPrimary < byteBuffer.length)
            {
                recordLength = byteBuffer[counterPrimary];
                while (recordLength == 0)
                {
                    counterPrimary ++;
                    if (counterPrimary == byteBuffer.length)
                    {
                        break;
                    }
                    recordLength = byteBuffer[counterPrimary];
                }
                if (counterPrimary == byteBuffer.length)
                {
                    break;
                }
                long firstSectAddr = 0;
                firstSectAddr = byteBuffer[counterPrimary + 2] & 0xff;
                firstSectAddr = (((byteBuffer[counterPrimary + 3] << 8) & 0xffff) | firstSectAddr);
                firstSectAddr = (((byteBuffer[counterPrimary + 4] << 16) & 0xffffff) | firstSectAddr);
                firstSectAddr = (((byteBuffer[counterPrimary + 5] << 24) & 0xffffffff) | firstSectAddr);
                long pathLength = 0;
                pathLength = (byteBuffer[counterPrimary + 10] & 0xff);
                pathLength = (((byteBuffer[counterPrimary + 11] << 8) & 0xffff) | pathLength);
                pathLength = (((byteBuffer[counterPrimary + 12] << 16) & 0xffffff) | pathLength);
                pathLength = (((byteBuffer[counterPrimary + 13] << 24) & 0xffffffff) | pathLength);
                int nameLength = 0;
                if (((byteBuffer[counterPrimary + 25] >>> 1) % 2) == 1)
                {//dirs are as normal
                    nameLength = (byteBuffer[counterPrimary + 32] & 0xff);
                }
                else if (((byteBuffer[counterPrimary + 25] >>> 1) % 2) == 0)
                {//account for the ";1" in files
                    nameLength = (byteBuffer[counterPrimary + 32] & 0xff) - 2;
                }
                String name = "";
                for (int r = 0; r < nameLength; r++)
                {
                    name += String.valueOf(Character.toChars(new Integer(byteBuffer[counterPrimary + 33 + r] & 0xff)));
                }
                if (name.equals("\0"))
                {//root
                    ;//do nothing
                }
                else if (name.equals("\1"))
                {//current dir
                    ;//do nothing
                }
                else if (((byteBuffer[counterPrimary + 25] >>> 1) % 2) == 1)
                {//it's a dir
                    PS1Path tempDir = new PS1Path(name, firstSectAddr, pathLength, false);
                    nonPrimaryDirs.add(tempDir);
                }
                else if (((byteBuffer[counterPrimary + 25] >>> 1) % 2) == 0)
                {//it's a file
                    PS1Path tempFile = new PS1Path(name, firstSectAddr, pathLength, true);
                    discFiles.add(tempFile);
                }
                counterPrimary += recordLength;
            }

            for (PS1Path dir : nonPrimaryDirs)
            {//Do same as above, but for non-primary directory sectors
                long sectorAddress = dir.getAddressOfFirstSector();
                int dirLength = (int)dir.getPathLength();
                byteBuffer = new byte[dirLength];

                if (dirLength <= 2048)
                {
                    disc.seek((sectorAddress * 2352L) + 24L);
                    disc.readFully(byteBuffer);
                }
                else if (dirLength > 2048)
                {
                    int numOfSectors = dirLength / 2048;
                    for (int i = 0; i < numOfSectors; i++)
                    {
                        disc.seek((sectorAddress * 2352L) + 24L + (i * 2352));
                        disc.read(byteBuffer, (i * 2048), 2048);
                    }
                }

                int counterNonPrime = 0;
                while (counterNonPrime < byteBuffer.length)
                {
                    recordLength = byteBuffer[counterNonPrime];
                    while (recordLength == 0)
                    {
                        counterNonPrime ++;
                        if (counterNonPrime == byteBuffer.length)
                        {
                            break;
                        }
                        recordLength = byteBuffer[counterNonPrime];
                    }
                    if (counterNonPrime == byteBuffer.length)
                    {
                        break;
                    }
                    long PS1PathAddress = 0;
                    PS1PathAddress = byteBuffer[counterNonPrime + 2] & 0xff;
                    PS1PathAddress = (((byteBuffer[counterNonPrime + 3] << 8) & 0xffff) | PS1PathAddress);
                    PS1PathAddress = (((byteBuffer[counterNonPrime + 4] << 16) & 0xffffff) | PS1PathAddress);
                    PS1PathAddress = (((byteBuffer[counterNonPrime + 5] << 24) & 0xffffff) | PS1PathAddress);
                    long PS1PathLength = 0;
                    PS1PathLength = byteBuffer[counterNonPrime + 10] & 0xff;
                    PS1PathLength = (((byteBuffer[counterNonPrime + 11] << 8) & 0xffff) | PS1PathLength);
                    PS1PathLength = (((byteBuffer[counterNonPrime + 12] << 16) & 0xffffff) | PS1PathLength);
                    PS1PathLength = (((byteBuffer[counterNonPrime + 13] << 24) & 0xffffffff) | PS1PathLength);
                    int PS1PathNameLength = 0;
                    if (((byteBuffer[counterNonPrime + 25] >>> 1) % 2) == 1)
                    {//dirs are as normal
                        PS1PathNameLength = (byteBuffer[counterNonPrime + 32] & 0xff);
                    }
                    else if (((byteBuffer[counterNonPrime + 25] >>> 1) % 2) == 0)
                    {//account for the ";1" in files
                        PS1PathNameLength = (byteBuffer[counterNonPrime + 32] & 0xff) - 2;
                    }
                    String PS1PathName = "";
                    for (int r = 0; r < PS1PathNameLength; r++)
                    {//build String name
                        PS1PathName += String.valueOf(Character.toChars(new Integer(byteBuffer[counterNonPrime + 33 + r] & 0xff)));
                    }
                    if (PS1PathName.equals("\0"))
                    {//root
                        ;
                    }
                    else if (PS1PathName.equals("\1"))
                    {//current dir
                        ;
                    }
                    else if (((byteBuffer[counterNonPrime + 25] >>> 1) % 2) == 1)
                    {// it's a dir - should NEVER enter this, check w/ breakpoint occasionally
                        PS1Path tempPS1PathDir = new PS1Path(PS1PathName, PS1PathAddress, PS1PathLength, false);
                        nonPrimaryDirs.add(tempPS1PathDir);
                    }
                    else if (((byteBuffer[counterNonPrime + 25] >>> 1) % 2) == 0)
                    {//it's a file
                        PS1Path tempPS1PathFile = new PS1Path(PS1PathName, PS1PathAddress, PS1PathLength, true);
                        discFiles.add(tempPS1PathFile);
                    }
                    counterNonPrime += recordLength;
                }
            }
            result = true;
        }
        catch(Exception e)
        {
            throw e;
        }
        finally
        {
            return result;
        }
    }

    /**
     * Method to load data from a disc image into various data structures.
     * @param   File discImage
     * @return  boolean result
     */
    public static boolean readFromDiscImage(File discImage)
    {
        RandomAccessFile disc = null;
        boolean result = false;
        try
        {//don't forget to close RandomAccessFile disc when you're done
            disc = new RandomAccessFile(discImage, "r");

            readDirectories(disc);
            for (FixedSizeDataTable table : dataTables)
            {
                for (PS1Path file : discFiles)
                {
                    if (table.getFileName().equals(file.getPathName()))
                    {
                        long sectorAddress = file.getAddressOfFirstSector();
                        disc.seek((sectorAddress * 2352) + 18);
                        int submodeInt = disc.readUnsignedByte();

                        long tableAddress = table.getTableAddress();

                        int counterRead = 0;
                        long tableAddressOffset = 0;
                        while (counterRead < tableAddress)
                        {//this block navigates to first sector of table 
                            if ((submodeInt & 0x20) != 0x20)
                            {//form 1: 2048 byte data field 
                                if ((counterRead + 2048) <= tableAddress)
                                {//if tableAddress is not within this sector
                                    counterRead += 2048;
                                    tableAddressOffset ++;
                                }
                                else
                                {//if tableAddress is within this sector
                                    break;
                                }
                            }
                            else if ((submodeInt & 0x20) == 0x20)
                            {//form 2: 2324 byte data field
                                if ((counterRead + 2324) <= tableAddress)
                                {//if tableAddress is not within this sector
                                    counterRead += 2324;
                                    tableAddressOffset ++;
                                }
                                else
                                {// if tableAddress is within this sector
                                    break;
                                }
                            }
                            disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 18);
                            submodeInt = disc.readUnsignedByte();
                        }
                        //if you are here, you've reached the sector where the table starts
                        //ie, case 1: start of data table

                        //navigate to start of data table
                        disc.seek(((sectorAddress + tableAddressOffset) * 2352) + (tableAddress - counterRead) + 24);

                        //get size of the part of the data table that's in this sector
                        int bytesInThisSector = 0;
                        if ((submodeInt & 0x20) != 0x20)
                        {//form 1: 2048 byte data field
                            bytesInThisSector = 2048 - ((int)tableAddress - counterRead);
                        }
                        else if ((submodeInt & 0x20) == 0x20)
                        {//form 2: 2324 byte data field
                            bytesInThisSector = 2324 - ((int)tableAddress - counterRead);
                        }

                        //read data in first sector into byteBuffer
                        int tableSize = table.getTableLength();

                        //what if table is smaller than sector size? need to account for that?
                        if (tableSize < bytesInThisSector)
                        {//if table size is < number of bytes left in this, the first sector...
                            bytesInThisSector = tableSize;
                        }//then read only tableSize number of bytes.
                        byte[] byteBuffer = new byte[tableSize];
                        disc.read(byteBuffer, 0, bytesInThisSector);
                        tableAddressOffset ++;

                        //would be a really good idea to turn this into a method, doing the above with a
                        //conditional based on yet another counter

                        int bytesRead = bytesInThisSector;

                        while (bytesRead < tableSize)
                        {
                            //later, when you convert this to a method call, add case 1 
                            //(and setup) to this while loop using a conditional
                            disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 18);
                            submodeInt = disc.readUnsignedByte();
                            if ((submodeInt & 0x20) != 0x20)
                            {//form 1: 2048 byte data field
                                if ((tableSize - bytesRead) >= 2048)
                                {//case 2: middle of data table
                                    disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 24);
                                    disc.read(byteBuffer, bytesRead, 2048);//print off table here
                                    bytesRead += 2048;
                                    tableAddressOffset ++;
                                }
                                else if ((tableSize - bytesRead) < 2048)
                                {//case 3: end of data table
                                    disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 24);
                                    disc.read(byteBuffer, bytesRead, (tableSize - bytesRead));
                                    bytesRead += 2048;
                                    tableAddressOffset ++;
                                }
                            }
                            else if ((submodeInt & 0x20) == 0x20)
                            {//form 2: 2324 byte data field
                                if ((tableSize - bytesRead) >= 2048)
                                {//case 2: middle of data table
                                    disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 24);
                                    disc.read(byteBuffer, bytesRead, 2324);
                                    bytesRead += 2324;
                                    tableAddressOffset ++;
                                }
                                else if ((tableSize - bytesRead) < 2048)
                                {//case 3: end of data table
                                    disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 24);
                                    disc.read(byteBuffer, bytesRead, (tableSize - bytesRead));
                                    bytesRead += 2324;
                                    tableAddressOffset ++;
                                }
                            }//^maybe change both of the above to read only correct amount?

                        }

                        table.data = new HashMap<String,Integer[]>();
                        int entrySize = table.getEntrySize();
                        int numOfEntries = table.getNumOfEntries();

                        int buffOffset = 0;
                        for (int i = 0; i < numOfEntries; i++)
                        {
                            Integer[] intBuffer = new Integer[entrySize];
                            for (int j = 0; j < entrySize; j++)
                            {
                                int tempInt = (byteBuffer[buffOffset] & 0xff);
                                intBuffer[j] = tempInt;
                                buffOffset ++;
                            }
                            //abstract the code below by putting the name of the string array in each data.csv entry
                            if (table.getTableName().equals("diffTables"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i], intBuffer);
                            }
                            if (table.getTableName().equals("items1"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_ITEMS[i], intBuffer);
                            }
                            if (table.getTableName().equals("mysticAbsorbs"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_ENEMIES[i], intBuffer);
                            }
                            if (table.getTableName().equals("monsterForms1"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i], intBuffer);
                            }
                            if (table.getTableName().equals("monsterForms2"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[i], intBuffer);
                            }
                            if (table.getTableName().equals("skills1"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_ABILITIES[i], intBuffer);
                            }
                            if (table.getTableName().equals("actionGrowths"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_ACTIONS[i], intBuffer);
                            }
                            if (table.getTableName().equals("magicLearning"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_MAGIC[i], intBuffer);
                            }
                            if (table.getTableName().equals("gunLearning"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_GUN_TECHS[i], intBuffer);
                            }
                            if (table.getTableName().equals("mecSelfDev"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_UNSIGNED_QUARTER_BYTE[i], intBuffer);
                            }
                            if (table.getTableName().equals("initialCharacter1"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_CHARACTERS[i], intBuffer);
                            }
                            if (table.getTableName().equals("initialCharacter2"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_CHARACTERS[i], intBuffer);
                            }
                            if (table.getTableName().equals("initialCharacter3"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_CHARACTERS[i], intBuffer);
                            }
                            if (table.getTableName().equals("initialCharacter4"))
                            {
                                table.data.put(DataHandler.ARRAY_STRING_CHARACTERS[i], intBuffer);
                            }
                        }
                        break;
                    }
                }
            }//thoughts for later: files with same name in diff dirs?
            //keep directory name in data table entry somehow?
            result = true;
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            try
            {
                if (disc != null)
                {
                    disc.close();
                }
            }
            catch (Exception e) {}
            return result;
        }
    }

    /**
     * Write changes to disc image.
     * @param   File discimage
     * @return  boolean result
     */
    public static boolean writeToDiscImage(File discImage)
    {
        RandomAccessFile disc = null;
        boolean result = false;
        try
        {
            disc = new RandomAccessFile(discImage, "rw");
            readDirectories(disc);
            for (FixedSizeDataTable table : dataTables)
            {
                for (PS1Path file : discFiles)
                {
                    if (table.getFileName().equals(file.getPathName()))
                    {
                        long sectorAddress = file.getAddressOfFirstSector();
                        disc.seek((sectorAddress * 2352) + 18);
                        int submodeInt = disc.readUnsignedByte();

                        long tableAddress = table.getTableAddress();
                        int tableSize = table.getTableLength();

                        int counterRead = 0;
                        long tableAddressOffset = 0; //may not be needed, depends what this tracks
                        while (counterRead < tableAddress)
                        {//this block navigates to the first sector of table, sector by sector
                            if ((submodeInt & 0x20) != 0x20)
                            {//form 1: 2048 byte data field
                                if ((counterRead + 2048) <= tableAddress)
                                {//if tableAddress is not within this sector
                                    counterRead += 2048;
                                    tableAddressOffset ++;
                                }
                                else
                                {//if tableAddress is within this sector, exit while loop
                                    break;
                                }
                            }
                            else if ((submodeInt & 0x20) == 0x20)
                            {//form 2: 2324 bye data field
                                if ((counterRead + 2342) <= tableAddress)
                                {//if address is not within this sector
                                    counterRead += 2324;
                                    tableAddressOffset ++;
                                }
                                else
                                {//if tableAddress is within this sector, exit while loop
                                    break;
                                }
                            }

                            disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 18);
                            submodeInt = disc.readUnsignedByte();
                        }
                        //if you are here, you've reached the sector where the table starts
                        //ie, case 1: start of data table

                        //first decompose HashMap<String, Integer[]> into larger int[] or Integer[]
                        int convertedArr[] = new int[tableSize];
                        if (table instanceof FixedSizeDataTable)
                        {//change to a case switch statement as you add types of data tables
                            int numOfEntries = table.getNumOfEntries();
                            int entrySize = table.getEntrySize();
                            for (int cnvrtOtr = 0; cnvrtOtr < numOfEntries; cnvrtOtr ++)
                            {
                                Integer[] tempDataArr = new Integer[entrySize];//all null? why? it works though
                                //abstract the code below by putting the correct string array in each data.csv entry
                                if (table.getTableName().equals("diffTables"))
                                {//change to a case-switch statement when you add more data tables
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("items1"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_ITEMS[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("mysticAbsorbs"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_ENEMIES[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("monsterForms1"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("monsterForms2"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_UNSIGNED_BYTE[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("skills1"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_ABILITIES[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("actionGrowths"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_ACTIONS[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("magicLearning"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_MAGIC[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("gunLearning"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_GUN_TECHS[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("mecSelfDev"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_UNSIGNED_QUARTER_BYTE[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("initialCharacter1"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_CHARACTERS[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("initialCharacter2"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_CHARACTERS[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("initialCharacter3"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_CHARACTERS[cnvrtOtr]);
                                }
                                else if (table.getTableName().equals("initialCharacter4"))
                                {
                                    tempDataArr = table.data.get(DataHandler.ARRAY_STRING_CHARACTERS[cnvrtOtr]);
                                }
                                for (int cnvrtInnr = 0; cnvrtInnr < entrySize; cnvrtInnr ++)
                                {
                                    convertedArr[(cnvrtOtr * entrySize) + cnvrtInnr] = tempDataArr[cnvrtInnr];
                                }
                            }
                        }

                        //navigate to start of subheader field in sector

                        disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 18);
                        submodeInt = disc.readUnsignedByte();
                        long[] writeToSector = null;
                        long firstSectorWriteOffset = tableAddress - counterRead;
                        long tableBytesWritten = 0;
                        if ((submodeInt & 0x20) != 0x20)
                        {//form 1: 2048 byte data field + other things
                            writeToSector = new long[2060];
                            tableBytesWritten = 2048 - firstSectorWriteOffset;//works!
                        }
                        else if ((submodeInt & 0x20) == 0x20)
                        {//form 2: 2324 byte data field + other things
                            writeToSector = new long[2332];
                            tableBytesWritten = 2332 - firstSectorWriteOffset;
                            //works? not tested yet.
                        }
                        disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16);

                        //error is below here.

                        //read first part of sector (pre-data-table) into array writeToSector
                        int sctCtr = 0;
                        for (; sctCtr < (firstSectorWriteOffset + 8); sctCtr ++)//8 for the subheader
                        {
                            writeToSector[sctCtr] = (long)disc.readUnsignedByte();
                        }
                        if ((submodeInt & 0x20) != 0x20)
                        {//form 1: 2048 byte data field + 8 byte header
                            for (int outputCtr = 0; sctCtr < 2056; sctCtr++, outputCtr++)
                            {
                                if (outputCtr < tableSize)
                                {
                                    writeToSector[sctCtr] = (long)convertedArr[outputCtr];
                                }
                                if (outputCtr == tableSize)
                                {
                                    disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16 + sctCtr);
                                    writeToSector[sctCtr] = (long)disc.readUnsignedByte();
                                }
                                if (outputCtr > tableSize)
                                {
                                    writeToSector[sctCtr] = (long)disc.readUnsignedByte();
                                    //line 868, need to seek to outputCtr + sctCtr before you start to read... 
                                    //need it to be a third conditional where outputCtr == tableSize maybe, so 
                                    //it's only called once.
                                    //may need to move increments for outputCtr to ensure it doesn't count twice 
                                    //in several cases, like when sctCtr and outputCtr are added together.
                                    //actually you can't use sctCtr + outputCtr, you need to use something else
                                    //wait, you only use sctCtr + outputCtr once, when it's the equals case (at 
                                    //the end of the data table when you need to seek to the end of the data 
                                    //table), so it's probably fine.

                                    //cases:
                                    //1. before you reach data table (already functioning)
                                    //2. during the data table (sort of functions)
                                    //3. after the data table (malfunctions)
                                }
                                //all of the above code works flawlessly.
                                //i have eliminated the areas above as the source of the bug.
                                //next up i need to print some stuff below.
                            }
                        }
                        else if ((submodeInt & 0x20) == 0x20)
                        {//form 2: 2324 bye data field + 8 byte header
                            for (int outputCtr = 0; sctCtr < 2332; sctCtr++, outputCtr++)
                            {
                                if (outputCtr < tableSize)
                                {
                                    writeToSector[sctCtr] = (long)convertedArr[sctCtr];
                                }
                                if (outputCtr == tableSize)
                                {
                                    disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16 +sctCtr);
                                    writeToSector[sctCtr] = (long)disc.readUnsignedByte();
                                }
                                if (outputCtr > tableSize)
                                {
                                    writeToSector[sctCtr] = (long)disc.readUnsignedByte();
                                }
                            }
                        }
                        EDC_ECC_Generator edc_ecc_gen = new EDC_ECC_Generator();
                        if ((submodeInt & 0x20) != 0x20)
                        {//form 1: 8 byte subheader + 2048 byte data field
                            writeToSector = edc_ecc_gen.get_edc_ecc(writeToSector);
                        }
                        else if ((submodeInt & 0x20) == 0x20)
                        {//form 2: 8 byte subheader + 2324 byte data field
                            writeToSector = edc_ecc_gen.get_edc_form2(writeToSector);
                        }

                        disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16);
                        for (int writeCtr = 4; writeCtr < writeToSector.length; writeCtr++)
                        {//this loop works because get_edc_ecc() returns corrected array
                            disc.write((int)(writeToSector[writeCtr] & 0xFF));

                            //conclusion: it's flawless, error must be in where it's being
                            //written. add addresses to print statements in base 10, and
                            //rip this sector in same using wxHexEditor.
                        }//I think this is the offending 
                        //now update all the necessary fields
                        tableAddressOffset ++;
                        //items write correctly, but mystic absorbs don't because the latter is after the former in this sector.
                        //you never read the rest of the bytes in this sector into the array that's passed to the edc/ecc generator

                        //error is NOT above here, it's below here!

                        //stopped here
                        //case 2: middle of data table
                        disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 18);
                        submodeInt = disc.readUnsignedByte();

                        int nextSectorSize;
                        disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16);
                        while(tableBytesWritten < tableSize)
                        {//remainder of the i/o - cases 2 and 3
                            //repeat above portion, but only read the subheader
                            // and whatever else is needed
                            if (tableAddressOffset == 20)
                            {
                                int i = 0;
                                i++;
                                i++;
                            }
                            if ((submodeInt & 0x20) != 0x20)
                            {
                                writeToSector = new long[2060];
                                nextSectorSize = 2048;//12 smaller than above
                            }
                            else// if ((submodeInt & 0x20) == 0x20)
                            {
                                writeToSector = new long[2332];
                                nextSectorSize = 2324;//8 smaller than above
                            }
                            if ((tableSize - tableBytesWritten) > nextSectorSize)
                            {//case 2: middle of data table
                                int sctCntr;
                                //System.out.println("Case 2: Read" + "\n");
                                for (sctCntr = 0; sctCntr < 8; sctCntr ++)
                                {//read subheader into writeToSector.
                                    //problem is the last seek at the end of this while statement!
                                    writeToSector[sctCntr] = (long)disc.readUnsignedByte();
                                    //System.out.print("Read Address: ");
                                    //long tempAddress = ((sectorAddress + tableAddressOffset) * 2352) + 16 + sctCntr;
                                    //System.out.print(Long.toHexString(tempAddress));
                                    //System.out.print(", \t Value: " +  writeToSector[sctCntr] + "\n");
                                }
                                for (; sctCntr < (nextSectorSize + 8); sctCntr ++, tableBytesWritten ++)
                                {//i think the error is here, or rather where nextSectorSize is initialized...
                                    writeToSector[sctCntr] = convertedArr[(int)tableBytesWritten];
                                }//why is tableBytesWritten a long?  should be an int.
                                if ((submodeInt & 0x20) != 0x20)
                                {//form 1: 8 byte subheader + 2048 byte data field
                                    writeToSector = edc_ecc_gen.get_edc_ecc(writeToSector);
                                }
                                else if ((submodeInt & 0x20) == 0x20)
                                {//form 1: 8 byte subheader + 2324 byte data field
                                    writeToSector = edc_ecc_gen.get_edc_form2(writeToSector);
                                }
                                disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16);
                                //System.out.println("\nCase 2: Write" + "\n");
                                for (int writeCtr = 4; writeCtr < writeToSector.length; writeCtr ++)
                                {//this loop works because get_edc_ecc() returns array of correct length
                                    disc.write((int)(writeToSector[writeCtr] & 0xFF));
                                    //System.out.print("Write Address: ");
                                    //long tempAddress = ((sectorAddress + tableAddressOffset) * 2352) + 16 + writeCtr - 4;
                                    //System.out.print(Long.toHexString(tempAddress));
                                    //System.out.print(", \t Value: " +  writeToSector[writeCtr] + "\n");
                                }
                                if ((submodeInt & 0x20) != 0x20)
                                {
                                    //shouldn't need to increment tableBytesWritten, as it's incremented above.
                                    //tableBytesWritten += nextSectorSize;
                                    tableAddressOffset ++;
                                }
                                else// if ((submodeInt & 0x20) == 0x20)
                                {
                                    //tableBytesWritten += nextSectorSize;
                                    tableAddressOffset ++;
                                }
                                disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 18);
                                submodeInt = disc.readUnsignedByte();
                                disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16);
                            }
                            //case 2 works 100% correct, now on to case 3
                            else if ((tableSize - tableBytesWritten) < nextSectorSize)
                            {//case 3: last sector of data table, be sure to include a break to escape the loop
                                int sctCntr;
                                for (sctCntr = 0; sctCntr < 8; sctCntr ++)
                                {//read subheader into writeToSector
                                    writeToSector[sctCntr] = (long)disc.readUnsignedByte();
                                }
                                while (tableBytesWritten < tableSize)//this may need to be an <=, i'm not sure.
                                {//get remaining bytes from convertedArr into correct positions in writeToSector
                                    writeToSector[sctCntr] = convertedArr[(int)tableBytesWritten];
                                    sctCntr ++;
                                    tableBytesWritten ++;
                                }
                                disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16 + sctCntr);
                                while (sctCntr < (nextSectorSize + 8))
                                {//read rest of sector from disc image into writeToSector
                                    writeToSector[sctCntr] = (long)disc.readUnsignedByte();
                                    //long tempLong = writeToSector[sctCntr];
                                    //System.out.println(Long.toHexString(tempLong));
                                    sctCntr ++;

                                }
                                //print statements here
                                //System.out.println();
                                //for (int i = 0; i < writeToSector.length; i++)
                                //{
                                //    if (writeToSector[i] < 16)
                                //    {
                                //        System.out.print("0" + Long.toHexString(writeToSector[i]) + "-");
                                //    }
                                //    else
                                //    {
                                //        System.out.print(Long.toHexString(writeToSector[i]) + "-");
                                //    }
                                //    if ((i != 0) && (i % 16 == 0))
                                //    {
                                //        System.out.println();
                                //    }
                                //}
                                //System.out.println();
                                //conclusion of above print statements: error is definitely here
                                //check if it's reading incorrect values or not reading at all
                                //do this by attacking print statments to above 2 sections of code:
                                //section for remainder of data table, and section for rest of sector.
                                //also does this bug pop up in all data tables, or just mystic absorbs?
                                //for some reason item data never enters this area...
                                if ((submodeInt & 0x20) != 0x20)
                                {
                                    writeToSector = edc_ecc_gen.get_edc_ecc(writeToSector);
                                }
                                else if ((submodeInt & 0x20) == 0x20)
                                {
                                    writeToSector = edc_ecc_gen.get_edc_form2(writeToSector);
                                }
                                //think we need a seek right here... check above.
                                disc.seek(((sectorAddress + tableAddressOffset) * 2352) + 16);
                                for (int writeCtr = 4; writeCtr < writeToSector.length; writeCtr ++)
                                {
                                    disc.write((int)(writeToSector[writeCtr] & 0xFF));
                                }
                                tableAddressOffset ++; //not needed.
                                break;
                            }
                            break;
                        }
                    }
                }
            }
            result = true;
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            try
            {
                if (disc != null)
                {
                    disc.close();
                }
            }
            catch (Exception e) {}
            return result;
        }
    }
}
