import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileNameExtensionFilter;
/** 
 * SnakeOil (c) 2015 - 2019
 * https://bitbucket.org/MysticLord/
 */
public class GUI extends JFrame
{
    private Container contentPane;

    private JMenuBar menuBar;
    private JMenu fileMenu;
    private JMenuItem newPatch;
    private JMenuItem openPatch;
    private JMenuItem savePatch;
    private JMenuItem closePatch;
    private JMenuItem readDiscImage;
    private JMenuItem writeDiscImage;

    private JButton diffsButton;
    private JButton itemsButton;
    private JButton mysticAbsorbsButton;
    private JButton monsterForms1Button;
    private JButton monsterForms2Button;
    private JButton skills1Button;
    private JButton actionGrowthsButton;
    private JButton magicLearningButton;
    private JButton gunTechLearningButton;
    private JButton mecDevButton;
    private JButton initChar1Button;
    private JButton initChar2Button;
    private JButton initChar3Button;
    private JButton initChar4Button;

    private DiffTables diffsFrame;
    private Items itemsFrame;
    private MysticAbsorbs mysticAbsorbsFrame;
    private MonsterForms1 monsterForms1Frame;
    private MonsterForms2 monsterForms2Frame;
    private Skills skills1Frame;
    private ActionGrowths actionGrowthsFrame;
    private MagicLearning magicLearningFrame;
    private GunLearning gunTechLearningFrame;
    private MecSelfDev mecDevFrame;
    private InitChar initChar1Frame;
    private InitChar initChar2Frame;
    private InitChar initChar3Frame;
    private InitChar initChar4Frame;

    /**
     * Constructor for objects of class GUI
     */
    public GUI()
    {
        super("SnakeOil");
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

        contentPane = getContentPane();
        contentPane.setLayout(new GridLayout(4,4,1,1)); //GridLayout(rows, columns);

        makeMenu();

        makeButtons();

        pack();
        setVisible(true);
        setResizable(false);

        savePatch.setEnabled(false);
        closePatch.setEnabled(false);
        mysticAbsorbsButton.setEnabled(false);
        writeDiscImage.setEnabled(false);

        diffsButton.setEnabled(false);
        itemsButton.setEnabled(false);
        mysticAbsorbsButton.setEnabled(false);
        monsterForms1Button.setEnabled(false);
        monsterForms2Button.setEnabled(false);
        skills1Button.setEnabled(false);
        actionGrowthsButton.setEnabled(false);
        magicLearningButton.setEnabled(false);
        gunTechLearningButton.setEnabled(false);
        mecDevButton.setEnabled(false);
        initChar1Button.setEnabled(false);
        initChar2Button.setEnabled(false);
        initChar3Button.setEnabled(false);
        initChar4Button.setEnabled(false);

        //System.out.println("Height: " + getHeight() + ", Width: " + getWidth());

        DataHandler.readFromPatch("basePatch.snakeoil");
    }

    /**
     * Method to set up menu.
     */
    private void makeMenu()
    {
        menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        fileMenu = new JMenu("File");
        menuBar.add(fileMenu);

        newPatch = new JMenuItem("Make a new patch");
        newPatch.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    newPatch.setEnabled(false);
                    openPatch.setEnabled(false);
                    savePatch.setEnabled(false);
                    closePatch.setEnabled(false);
                    readDiscImage.setEnabled(false);
                    writeDiscImage.setEnabled(false);
                    diffsButton.setEnabled(false);
                    itemsButton.setEnabled(false);
                    mysticAbsorbsButton.setEnabled(false);
                    monsterForms1Button.setEnabled(false);
                    monsterForms2Button.setEnabled(false);
                    skills1Button.setEnabled(false);
                    actionGrowthsButton.setEnabled(false);
                    magicLearningButton.setEnabled(false);
                    gunTechLearningButton.setEnabled(false);
                    mecDevButton.setEnabled(false);
                    initChar1Button.setEnabled(false);
                    initChar2Button.setEnabled(false);
                    initChar3Button.setEnabled(false);
                    initChar4Button.setEnabled(false);

                    boolean readPatch = DataHandler.readFromPatch("basePatch.snakeoil");
                    if (readPatch)
                    {//case 1: readFromPatch works
                        newPatch.setEnabled(true);
                        openPatch.setEnabled(true);
                        savePatch.setEnabled(true);
                        closePatch.setEnabled(true);
                        readDiscImage.setEnabled(true);
                        writeDiscImage.setEnabled(true);
                        diffsButton.setEnabled(true);
                        itemsButton.setEnabled(true);
                        mysticAbsorbsButton.setEnabled(true);
                        monsterForms1Button.setEnabled(true);
                        monsterForms2Button.setEnabled(true);
                        skills1Button.setEnabled(true);
                        actionGrowthsButton.setEnabled(true);
                        magicLearningButton.setEnabled(true);
                        gunTechLearningButton.setEnabled(true);
                        mecDevButton.setEnabled(true);
                        initChar1Button.setEnabled(true);
                        initChar2Button.setEnabled(true);
                        initChar3Button.setEnabled(true);
                        initChar4Button.setEnabled(true);
                    }
                    else
                    {//case 2: readFromPatch fails
                        String errorString = "Make sure that basePatch.snakeoil and " +
                            "all other configuration files\nare in the following " + 
                            "folder:\n" + System.getProperty("user.dir");
                        JOptionPane.showMessageDialog(newPatch, errorString);
                        newPatch.setEnabled(true);
                        openPatch.setEnabled(true);
                        readDiscImage.setEnabled(true);
                    }
                }
            });
        fileMenu.add(newPatch);

        openPatch = new JMenuItem("Open an existing patch");
        openPatch.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    JFileChooser patchOpenChooser = new JFileChooser();

                    FileNameExtensionFilter patchOpenFilter = new FileNameExtensionFilter("*.snakeoil files only", "snakeoil");
                    patchOpenChooser.setFileFilter(patchOpenFilter);

                    patchOpenChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
                    int validFile = patchOpenChooser.showOpenDialog(GUI.this);

                    if (validFile == JFileChooser.APPROVE_OPTION)
                    {
                        String patchOpenFile = patchOpenChooser.getSelectedFile().toString();

                        newPatch.setEnabled(false);
                        openPatch.setEnabled(false);
                        savePatch.setEnabled(false);
                        closePatch.setEnabled(false);
                        readDiscImage.setEnabled(false);
                        writeDiscImage.setEnabled(false);
                        diffsButton.setEnabled(false);
                        itemsButton.setEnabled(false);
                        mysticAbsorbsButton.setEnabled(false);
                        monsterForms1Button.setEnabled(false);
                        monsterForms2Button.setEnabled(false);
                        skills1Button.setEnabled(false);
                        actionGrowthsButton.setEnabled(false);
                        magicLearningButton.setEnabled(false);
                        gunTechLearningButton.setEnabled(false);
                        mecDevButton.setEnabled(false);
                        initChar1Button.setEnabled(false);
                        initChar2Button.setEnabled(false);
                        initChar3Button.setEnabled(false);
                        initChar4Button.setEnabled(false);

                        boolean readPatch = DataHandler.readFromPatch(patchOpenFile);
                        if (readPatch)
                        {//case 1: readFromPatch works
                            newPatch.setEnabled(true);
                            openPatch.setEnabled(true);
                            savePatch.setEnabled(true);
                            closePatch.setEnabled(true);
                            readDiscImage.setEnabled(true);
                            writeDiscImage.setEnabled(true);
                            diffsButton.setEnabled(true);
                            itemsButton.setEnabled(true);
                            mysticAbsorbsButton.setEnabled(true);
                            monsterForms1Button.setEnabled(true);
                            monsterForms2Button.setEnabled(true);
                            skills1Button.setEnabled(true);
                            actionGrowthsButton.setEnabled(true);
                            magicLearningButton.setEnabled(true);
                            gunTechLearningButton.setEnabled(true);
                            mecDevButton.setEnabled(true);
                            initChar1Button.setEnabled(true);
                            initChar2Button.setEnabled(true);
                            initChar3Button.setEnabled(true);
                            initChar4Button.setEnabled(true);
                        }
                        else
                        {//case 2: readFromPatch fails
                            String errorString = "Make sure that basePatch.snakeoil and " + 
                                "all other configuration files\narein the following " +
                                "folder:\n" + System.getProperty("user.dir");
                            JOptionPane.showMessageDialog(openPatch, errorString);
                            newPatch.setEnabled(true);
                            openPatch.setEnabled(true);
                            readDiscImage.setEnabled(true);
                        }
                    }
                }
            });
        fileMenu.add(openPatch);

        savePatch = new JMenuItem("Save this patch");
        savePatch.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    JFileChooser patchSaveChooser = new JFileChooser();

                    FileNameExtensionFilter patchSaveFilter = new FileNameExtensionFilter("*.snakeoil files only", "snakeoil");
                    patchSaveChooser.setFileFilter(patchSaveFilter);

                    patchSaveChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
                    int validFile = patchSaveChooser.showSaveDialog(GUI.this);

                    if (validFile == JFileChooser.APPROVE_OPTION)
                    {
                        String patchSaveFile = patchSaveChooser.getSelectedFile().toString();
                        if (patchSaveFile.endsWith(".snakeoil") != true)
                        {
                            patchSaveFile = patchSaveFile + ".snakeoil";
                        }
                        //System.out.println(patchSaveChooser.getSelectedFile().toString());

                        newPatch.setEnabled(false);
                        openPatch.setEnabled(false);
                        savePatch.setEnabled(false);
                        closePatch.setEnabled(false);
                        readDiscImage.setEnabled(false);
                        writeDiscImage.setEnabled(false);
                        diffsButton.setEnabled(false);
                        itemsButton.setEnabled(false);
                        mysticAbsorbsButton.setEnabled(false);
                        monsterForms1Button.setEnabled(false);
                        monsterForms2Button.setEnabled(false);
                        skills1Button.setEnabled(false);
                        actionGrowthsButton.setEnabled(false);
                        magicLearningButton.setEnabled(false);
                        gunTechLearningButton.setEnabled(false);
                        mecDevButton.setEnabled(false);
                        initChar1Button.setEnabled(false);
                        initChar2Button.setEnabled(false);
                        initChar3Button.setEnabled(false);
                        initChar4Button.setEnabled(false);

                        boolean writePatch = DataHandler.writeToPatch(patchSaveFile);
                        if (!writePatch)
                        {//case 1: writeToPatch fails
                            String errorString = "Make sure that basePatch.snakeoil and " + 
                                "all other configuration files\nare in the following " +
                                "folder:\n" + System.getProperty("user.dir");
                            JOptionPane.showMessageDialog(savePatch,errorString);
                        }
                        newPatch.setEnabled(true);
                        openPatch.setEnabled(true);
                        savePatch.setEnabled(true);
                        closePatch.setEnabled(true);
                        readDiscImage.setEnabled(true);
                        writeDiscImage.setEnabled(true);
                        diffsButton.setEnabled(true);
                        itemsButton.setEnabled(true);
                        mysticAbsorbsButton.setEnabled(true);
                        monsterForms1Button.setEnabled(true);
                        monsterForms2Button.setEnabled(true);
                        skills1Button.setEnabled(true);
                        actionGrowthsButton.setEnabled(true);
                        magicLearningButton.setEnabled(true);
                        gunTechLearningButton.setEnabled(true);
                        mecDevButton.setEnabled(true);
                        initChar1Button.setEnabled(true);
                        initChar2Button.setEnabled(true);
                        initChar3Button.setEnabled(true);
                        initChar4Button.setEnabled(true);
                    }
                }
            });
        fileMenu.add(savePatch);

        closePatch = new JMenuItem("Close this patch");
        closePatch.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    savePatch.setEnabled(false);
                    closePatch.setEnabled(false);
                    writeDiscImage.setEnabled(false);
                    diffsButton.setEnabled(false);
                    itemsButton.setEnabled(false);
                    mysticAbsorbsButton.setEnabled(false);
                    monsterForms1Button.setEnabled(false);
                    monsterForms2Button.setEnabled(false);
                    skills1Button.setEnabled(false);
                    actionGrowthsButton.setEnabled(false);
                    magicLearningButton.setEnabled(false);
                    gunTechLearningButton.setEnabled(false);
                    mecDevButton.setEnabled(false);
                    initChar1Button.setEnabled(false);
                    initChar2Button.setEnabled(false);
                    initChar3Button.setEnabled(false);
                    initChar4Button.setEnabled(false);
                    if (itemsFrame != null)
                    {
                        itemsFrame.setVisible(false);
                        itemsFrame = null;
                    }
                    if (mysticAbsorbsFrame != null)
                    {
                        mysticAbsorbsFrame.setVisible(false);
                        mysticAbsorbsFrame = null;
                    }
                    if (monsterForms1Frame != null)
                    {
                        monsterForms1Frame.setVisible(false);
                        monsterForms1Frame = null;
                    }
                    if (skills1Frame != null)
                    {
                        skills1Frame.setVisible(false);
                        skills1Frame = null;
                    }
                    if (actionGrowthsFrame != null)
                    {
                        actionGrowthsFrame.setVisible(false);
                        actionGrowthsFrame = null;
                    }
                    if (magicLearningFrame != null)
                    {
                        magicLearningFrame.setVisible(false);
                        magicLearningFrame = null;
                    }
                    if (gunTechLearningFrame != null)
                    {
                        gunTechLearningFrame.setVisible(false);
                        gunTechLearningFrame = null;
                    }
                    if (mecDevFrame != null)
                    {
                        mecDevFrame.setVisible(false);
                        mecDevFrame = null;
                    }
                    if (initChar1Frame != null)
                    {
                        initChar1Frame.setVisible(false);
                        initChar1Frame = null;
                    }
                    if (initChar2Frame != null)
                    {
                        initChar2Frame.setVisible(false);
                        initChar2Frame = null;
                    }
                    if (initChar3Frame != null)
                    {
                        initChar3Frame.setVisible(false);
                        initChar3Frame = null;
                    }
                    if (initChar4Frame != null)
                    {
                        initChar4Frame.setVisible(false);
                        initChar4Frame = null;
                    }
                }
            });
        fileMenu.add(closePatch);

        fileMenu.addSeparator();

        readDiscImage = new JMenuItem("Read from disc image");
        readDiscImage.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    JFileChooser discImageChooser = new JFileChooser();

                    FileNameExtensionFilter discImageFilter = new FileNameExtensionFilter("*.bin, *.img, *.iso", "bin", "img", "iso");
                    discImageChooser.setFileFilter(discImageFilter);

                    discImageChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
                    int returnValue = discImageChooser.showOpenDialog(GUI.this);

                    if (returnValue == JFileChooser.APPROVE_OPTION)
                    {
                        File discImage = discImageChooser.getSelectedFile();
                        boolean readDisc = false;
                        newPatch.setEnabled(false);
                        openPatch.setEnabled(false);
                        savePatch.setEnabled(false);
                        closePatch.setEnabled(false);
                        readDiscImage.setEnabled(false);
                        writeDiscImage.setEnabled(false);
                        diffsButton.setEnabled(false);
                        itemsButton.setEnabled(false);
                        mysticAbsorbsButton.setEnabled(false);
                        monsterForms1Button.setEnabled(false);
                        monsterForms2Button.setEnabled(false);
                        skills1Button.setEnabled(false);
                        actionGrowthsButton.setEnabled(false);
                        magicLearningButton.setEnabled(false);
                        gunTechLearningButton.setEnabled(false);
                        mecDevButton.setEnabled(false);
                        initChar1Button.setEnabled(false);
                        initChar2Button.setEnabled(false);
                        initChar3Button.setEnabled(false);
                        initChar4Button.setEnabled(false);

                        try
                        {
                            readDisc = DataHandler.readFromDiscImage(discImage);
                        }
                        catch (Exception e)
                        {
                            System.out.println("Exception thrown in method readFromDiscImage(...)");
                            e.printStackTrace();
                        }
                        if (readDisc)
                        {//case 1: readFromDiscImage works 
                            newPatch.setEnabled(true);
                            openPatch.setEnabled(true);
                            savePatch.setEnabled(true);
                            closePatch.setEnabled(true);
                            readDiscImage.setEnabled(true);
                            writeDiscImage.setEnabled(true);
                            diffsButton.setEnabled(true);
                            itemsButton.setEnabled(true);
                            mysticAbsorbsButton.setEnabled(true);
                            monsterForms1Button.setEnabled(true);
                            monsterForms2Button.setEnabled(true);
                            skills1Button.setEnabled(true);
                            actionGrowthsButton.setEnabled(true);
                            magicLearningButton.setEnabled(true);
                            gunTechLearningButton.setEnabled(true);
                            mecDevButton.setEnabled(true);
                            initChar1Button.setEnabled(true);
                            initChar2Button.setEnabled(true);
                            initChar3Button.setEnabled(true);
                            initChar4Button.setEnabled(true);
                        }
                        else
                        {//case 2: readFromDiscImage fails
                            String errorString = "Make sure that basePatch.snakeoil and " + 
                                "all other configuration files\nare in the following " +
                                "folder:\n" + System.getProperty("user.dir");
                            JOptionPane.showMessageDialog(readDiscImage,errorString);
                            newPatch.setEnabled(true);
                            openPatch.setEnabled(true);
                            readDiscImage.setEnabled(true);
                        }
                    }
                }
            });
        fileMenu.add(readDiscImage);

        writeDiscImage = new JMenuItem("Write to disc image");
        writeDiscImage.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    JFileChooser discImageChooser = new JFileChooser();

                    FileNameExtensionFilter discImageFilter = new FileNameExtensionFilter("*.bin, *.img, *.iso", "bin", "img", "iso");
                    discImageChooser.setFileFilter(discImageFilter);

                    discImageChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
                    int returnValue = discImageChooser.showOpenDialog(GUI.this);

                    if (returnValue == JFileChooser.APPROVE_OPTION)
                    {
                        File discImage = null;
                        boolean writeDisc = false;
                        newPatch.setEnabled(false);
                        openPatch.setEnabled(false);
                        savePatch.setEnabled(false);
                        closePatch.setEnabled(false);
                        readDiscImage.setEnabled(false);
                        writeDiscImage.setEnabled(false);
                        diffsButton.setEnabled(false);
                        itemsButton.setEnabled(false);
                        mysticAbsorbsButton.setEnabled(false);
                        monsterForms1Button.setEnabled(false);
                        monsterForms2Button.setEnabled(false);
                        skills1Button.setEnabled(false);
                        actionGrowthsButton.setEnabled(false);
                        magicLearningButton.setEnabled(false);
                        gunTechLearningButton.setEnabled(false);
                        mecDevButton.setEnabled(false);
                        initChar1Button.setEnabled(false);
                        initChar2Button.setEnabled(false);
                        initChar3Button.setEnabled(false);
                        initChar4Button.setEnabled(false);

                        try
                        {
                            discImage = discImageChooser.getSelectedFile();
                            writeDisc = DataHandler.writeToDiscImage(discImage);
                        }
                        catch (Exception e)
                        {
                            System.out.println("Exception thrown in method writeDataStructtoDiscImage(...)");
                            e.printStackTrace();
                        }
                        if (!writeDisc)
                        {//writeToDiscImage fails
                            String errorString = "Make sure that basePatch.snakeoil and " + 
                                "all other configuration files\nare in the following " +
                                "folder:\n" + System.getProperty("user.dir");
                            JOptionPane.showMessageDialog(writeDiscImage,errorString);
                        }
                        newPatch.setEnabled(true);
                        openPatch.setEnabled(true);
                        savePatch.setEnabled(true);
                        closePatch.setEnabled(true);
                        readDiscImage.setEnabled(true);
                        writeDiscImage.setEnabled(true);
                        diffsButton.setEnabled(true);
                        itemsButton.setEnabled(true);
                        mysticAbsorbsButton.setEnabled(true);
                        monsterForms1Button.setEnabled(true);
                        monsterForms2Button.setEnabled(true);
                        skills1Button.setEnabled(true);
                        actionGrowthsButton.setEnabled(true);
                        magicLearningButton.setEnabled(true);
                        gunTechLearningButton.setEnabled(true);
                        mecDevButton.setEnabled(true);
                        initChar1Button.setEnabled(true);
                        initChar2Button.setEnabled(true);
                        initChar3Button.setEnabled(true);
                        initChar4Button.setEnabled(true);
                    }
                }
            });
        fileMenu.add(writeDiscImage);
    }

    /**
     * Method to set up buttons, including event logic.
     */
    private void makeButtons()
    {//change these into menu items, in a new JMenu
        diffsButton = new JButton("Difficulty Tables");
        diffsButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (diffsFrame == null)
                    {
                        diffsFrame = new DiffTables();
                    }
                    else if ((diffsFrame != null) && (diffsFrame.isVisible() == false))
                    {
                        diffsFrame.setVisible(true);
                    }
                }
            });
        contentPane.add(diffsButton);

        itemsButton = new JButton("Items");
        itemsButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (itemsFrame == null)
                    {
                        itemsFrame = new Items();
                    }
                    else if ((itemsFrame != null) && (itemsFrame.isVisible() == false))
                    {
                        itemsFrame.setVisible(true);
                    }
                }
            });
        contentPane.add(itemsButton);

        mysticAbsorbsButton = new JButton("Mystic Absorbs");
        mysticAbsorbsButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (mysticAbsorbsFrame == null)
                    {
                        mysticAbsorbsFrame = new MysticAbsorbs();
                    }
                    else if ((mysticAbsorbsFrame != null) && (mysticAbsorbsFrame.isVisible() == false))
                    {
                        mysticAbsorbsFrame.setVisible(true);
                    }
                }
            });
        contentPane.add(mysticAbsorbsButton);

        monsterForms1Button = new JButton("Monster Forms 1");
        monsterForms1Button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (monsterForms1Frame == null)
                    {
                        monsterForms1Frame = new MonsterForms1();
                    }
                    else if ((monsterForms1Frame != null) && (monsterForms1Frame.isVisible() == false))
                    {
                        monsterForms1Frame.setVisible(true);
                    }
                }
            });
        contentPane.add(monsterForms1Button);

        monsterForms2Button = new JButton("Monster Forms 2");
        monsterForms2Button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (monsterForms2Frame == null)
                    {
                        monsterForms2Frame = new MonsterForms2();
                    }
                    else if ((monsterForms2Frame != null) && (monsterForms2Frame.isVisible() == false))
                    {
                        monsterForms2Frame.setVisible(true);
                    }
                }
            });
        contentPane.add(monsterForms2Button);

        skills1Button = new JButton("Skills 1");
        skills1Button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (skills1Frame == null)
                    {
                        skills1Frame = new Skills();
                    }
                    else if ((skills1Frame != null) && (skills1Frame.isVisible() == false))
                    {
                        skills1Frame.setVisible(true);
                    }
                }
            });
        contentPane.add(skills1Button);

        actionGrowthsButton = new JButton("Action Growths");
        actionGrowthsButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (actionGrowthsFrame == null)
                    {
                        actionGrowthsFrame = new ActionGrowths();
                    }
                    else if ((actionGrowthsFrame != null) && (actionGrowthsFrame.isVisible() == false))
                    {
                        actionGrowthsFrame.setVisible(true);
                    }
                }
            });
        contentPane.add(actionGrowthsButton);

        magicLearningButton = new JButton("Magic Learning");
        magicLearningButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (magicLearningFrame == null)
                    {
                        magicLearningFrame = new MagicLearning();
                    }
                    else if ((magicLearningFrame != null) && (magicLearningFrame.isVisible() == false))
                    {
                        magicLearningFrame.setVisible(true);
                    }
                }
            });
        contentPane.add(magicLearningButton);

        gunTechLearningButton = new JButton("Gun Tech Learning");
        gunTechLearningButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (gunTechLearningFrame == null)
                    {
                        gunTechLearningFrame = new GunLearning();
                    }
                    else if ((gunTechLearningFrame != null) && (gunTechLearningFrame.isVisible() == false))
                    {
                        gunTechLearningFrame.setVisible(true);
                    }
                }
            });
        contentPane.add(gunTechLearningButton);

        mecDevButton = new JButton("Mec Self-Dev Skills");
        mecDevButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (mecDevFrame == null)
                    {
                        mecDevFrame = new MecSelfDev();
                    }
                    else if ((mecDevFrame != null) && (mecDevFrame.isVisible() == false))
                    {
                        mecDevFrame.setVisible(true);
                    }
                }
            });
        contentPane.add(mecDevButton);

        initChar1Button = new JButton("Characters 1");
        initChar1Button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (initChar1Frame == null)
                    {
                        initChar1Frame = new InitChar(1);
                    }
                    else if ((initChar1Frame != null) && (initChar1Frame.isVisible() == false))
                    {
                        initChar1Frame.setVisible(true);
                    }
                }
            });
        contentPane.add(initChar1Button);
        
        initChar2Button = new JButton("Characters 2");
        initChar2Button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (initChar2Frame == null)
                    {
                        initChar2Frame = new InitChar(2);
                    }
                    else if ((initChar2Frame != null) && (initChar2Frame.isVisible() == false))
                    {
                        initChar2Frame.setVisible(true);
                    }
                }
            });
        contentPane.add(initChar2Button);
        
        initChar3Button = new JButton("Characters 3");
        initChar3Button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (initChar3Frame == null)
                    {
                        initChar3Frame = new InitChar(3);
                    }
                    else if ((initChar3Frame != null) && (initChar3Frame.isVisible() == false))
                    {
                        initChar3Frame.setVisible(true);
                    }
                }
            });
        contentPane.add(initChar3Button);

        initChar4Button = new JButton("Characters 4");
        initChar4Button.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (initChar4Frame == null)
                    {
                        initChar4Frame = new InitChar(4);
                    }
                    else if ((initChar4Frame != null) && (initChar4Frame.isVisible() == false))
                    {
                        initChar4Frame.setVisible(true);
                    }
                }
            });
        contentPane.add(initChar4Button);

    }

    public static void main (String[] args)
    {
        GUI SnakeOil = new GUI();
    }
}
