import java.awt.*;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.io.*;
import java.lang.String.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.BoxLayout;
import javax.swing.event.*;
import java.text.*;
import javax.swing.border.*;
import java.util.HashMap;
/**
 * Class to handle editing monster transformation data (table 2) for saga frontier.
 * SnakeOil (c) 2015 - 2019
 * https://bitbucket.org/MysticLord/
 */
public class MonsterForms2 extends JFrame
{

    //local references to static fields
    private HashMap<String, Integer[]> monsterDataStruct;
    private Integer[] currentEntryArray;

    //misc class fields
    private String currentEntryName;

    //container area
    private JScrollPane monsterScrollPane;
    private Box FrameBox;
    private JPanel BorderPanel;
    private JPanel FlowPanel;
    
    //secondary container area
    private Box firstBox;
    private Box secondBox;
    private Box thirdBox;

    //form selector area
    private Box formSelectorBox;
    private JList<String> formSelector;
    private JScrollPane formSelectorScrollPane;

    //misc area
    private Box miscBox;
    private JLabel requiredHPLabel;
    private JComboBox<Integer> requiredHP;
    private JLabel monsterFormLabel;
    private JComboBox monsterForm;
    
    //head area
    private JPanel headPanel;
    private JComboBox<String> headSkill1;
    private JComboBox<String> headSkill2;
    private JComboBox<String> headSkill3;
    private JComboBox<String> headSkill4;
    
    //arm area
    private JPanel armPanel;
    private JComboBox<String> armSkill1;
    private JComboBox<String> armSkill2;
    private JComboBox<String> armSkill3;
    private JComboBox<String> armSkill4;
    
    //leg area
    private JPanel legPanel;
    private JComboBox<String> legSkill1;
    private JComboBox<String> legSkill2;
    private JComboBox<String> legSkill3;
    private JComboBox<String> legSkill4;

    //body area
    private JPanel bodyPanel;
    private JComboBox<String> bodySkill1;
    private JComboBox<String> bodySkill2;
    private JComboBox<String> bodySkill3;
    private JComboBox<String> bodySkill4;
    
    //breath area
    private JPanel breathPanel;
    private JComboBox<String> breathSkill1;
    private JComboBox<String> breathSkill2;
    private JComboBox<String> breathSkill3;
    private JComboBox<String> breathSkill4;
    
    //magic area
    private JPanel magicPanel;
    private JComboBox<String> magicSkill1;
    private JComboBox<String> magicSkill2;
    private JComboBox<String> magicSkill3;
    private JComboBox<String> magicSkill4;
    
    
    public MonsterForms2()
    {
        super("Monster Forms, Table 2");
        
        for (FixedSizeDataTable dataTable : DataHandler.dataTables)
        {
            if (dataTable.getTableName().equals("monsterForms2"))
            {//never entering this point..., check data handler
                //find and link local reference
                monsterDataStruct = dataTable.data;
                break;//quit when you find it - no reason to keep searching.
            }
        }

        setupContainers();
        setupSelector();
        setupMiscArea();
        setupHeadArea();
        setupArmArea();
        setupLegArea();
        setupBodyArea();
        setupBreathArea();
        setupMagicArea();
        
        currentEntryArray = monsterDataStruct.get("0");
        
        initializeMonsterForms2();
        pack();
        formSelector.setSelectedIndex(0);
        setVisible(true);
    }
    
    private void setupContainers()
    {
        //essential, do not modify
        BorderPanel = new JPanel(new BorderLayout());
        FlowPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        FrameBox = new Box(BoxLayout.X_AXIS);

        BorderPanel.add(FrameBox, BorderLayout.NORTH);
        FlowPanel.add(BorderPanel);

        monsterScrollPane = new JScrollPane(FlowPanel, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        getContentPane().add(monsterScrollPane);
        
        firstBox = new Box(BoxLayout.Y_AXIS);
        FrameBox.add(firstBox);
        
        secondBox = new Box(BoxLayout.Y_AXIS);
        FrameBox.add(secondBox);

        thirdBox = new Box(BoxLayout.Y_AXIS);
        FrameBox.add(thirdBox);
    }
    
    private void setupSelector()
    {
        //form selector area
        formSelectorBox = new Box(BoxLayout.Y_AXIS);
        formSelectorBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED),"#", TitledBorder.CENTER, TitledBorder.CENTER));
        firstBox.add(formSelectorBox);
        
        formSelector = new JList<String>(DataHandler.ARRAY_STRING_UNSIGNED_HALF_BYTE);
        formSelector.setVisibleRowCount(24);
        formSelector.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        formSelector.addListSelectionListener(new ListSelectionListener()
            {
                public void valueChanged(ListSelectionEvent e)
                {
                    currentEntryName = (String)formSelector.getSelectedValue();
                    currentEntryArray = monsterDataStruct.get(currentEntryName);
                    initializeMonsterForms2();
                }
            });
        
        formSelectorScrollPane = new JScrollPane(formSelector, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        formSelectorBox.add(formSelectorScrollPane);
    }

    private void setupMiscArea()
    {
        miscBox = new Box(BoxLayout.X_AXIS);
        miscBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Misc", TitledBorder.CENTER, TitledBorder.CENTER));
        secondBox.add(miscBox);
        //change so it has 2 grids instead of 1
        
        JPanel leftPanel = new JPanel(new GridLayout(2,1,0,0));
        miscBox.add(leftPanel);
        
        JPanel rightPanel = new JPanel(new GridLayout(2,1,0,0));
        miscBox.add(rightPanel);
        
        requiredHPLabel = new JLabel("Required HP");
        requiredHPLabel.setToolTipText("Multiply by 5 to get real amount");
        leftPanel.add(requiredHPLabel);
        
        requiredHP = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        requiredHP.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[0] = (Integer)requiredHP.getSelectedItem();
                }
            });
        rightPanel.add(requiredHP);
       
        monsterFormLabel = new JLabel("Form");
        leftPanel.add(monsterFormLabel);
        
        monsterForm = new JComboBox<String>(DataHandler.ARRAY_STRING_ENEMIES);
        monsterForm.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[1] = (Integer)monsterForm.getSelectedIndex();
                }
            });
        rightPanel.add(monsterForm);
    }
    
    private void setupHeadArea()
    {
        headPanel = new JPanel(new GridLayout(4,1,1,1));
        headPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Head Skills", TitledBorder.CENTER, TitledBorder.CENTER));
        secondBox.add(headPanel);
        
        headSkill1 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        headSkill1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[2] = (Integer)headSkill1.getSelectedIndex();
                }
            });
        headPanel.add(headSkill1);
        
        headSkill2 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        headSkill2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[3] = (Integer)headSkill2.getSelectedIndex();
                }
            });
        headPanel.add(headSkill2);
        
        headSkill3 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        headSkill3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[4] = (Integer)headSkill3.getSelectedIndex();
                }
            });
        headPanel.add(headSkill3);
        
        headSkill4 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        headSkill4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[5] = (Integer)headSkill4.getSelectedIndex();
                }
            });
        headPanel.add(headSkill4);
    }
    
    private void setupArmArea()
    {
        armPanel = new JPanel(new GridLayout(4,1,1,1));
        armPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Arm Skills", TitledBorder.CENTER, TitledBorder.CENTER));
        secondBox.add(armPanel);
        
        armSkill1 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        armSkill1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[6] = (Integer)armSkill1.getSelectedIndex();
                }
            });
        armPanel.add(armSkill1);
        
        armSkill2 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        armSkill2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[7] = (Integer)armSkill2.getSelectedIndex();
                }
            });
        armPanel.add(armSkill2);
        
        armSkill3 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        armSkill3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[8] = (Integer)armSkill3.getSelectedIndex();
                }
            });
        armPanel.add(armSkill3);
        
        armSkill4 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        armSkill4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[9] = (Integer)armSkill4.getSelectedIndex();
                }
            });
        armPanel.add(armSkill4);
    }
    
    private void setupLegArea()
    {
        legPanel = new JPanel(new GridLayout(4,1,1,1));
        legPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Leg Skills", TitledBorder.CENTER, TitledBorder.CENTER));
        secondBox.add(legPanel);
        
        legSkill1 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        legSkill1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[10] = (Integer)legSkill1.getSelectedIndex();
                }
            });
        legPanel.add(legSkill1);
        
        legSkill2 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        legSkill2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[11] = (Integer)legSkill2.getSelectedIndex();
                }
            });
        legPanel.add(legSkill2);
        
        legSkill3 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        legSkill3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[12] = (Integer)legSkill3.getSelectedIndex();
                }
            });
        legPanel.add(legSkill3);
        
        legSkill4 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        legSkill4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[13] = (Integer)legSkill4.getSelectedIndex();
                }
            });
        legPanel.add(legSkill4);
    }
    
    private void setupBodyArea()
    {
        bodyPanel = new JPanel(new GridLayout(4,1,1,1));
        bodyPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Body Skills", TitledBorder.CENTER, TitledBorder.CENTER));
        thirdBox.add(bodyPanel);
        
        bodySkill1 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        bodySkill1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[14] = (Integer)bodySkill1.getSelectedIndex();
                }
            });
        bodyPanel.add(bodySkill1);
        
        bodySkill2 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        bodySkill2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[15] = (Integer)bodySkill2.getSelectedIndex();
                }
            });
        bodyPanel.add(bodySkill2);
        
        bodySkill3 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        bodySkill3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[16] = (Integer)bodySkill3.getSelectedIndex();
                }
            });
        bodyPanel.add(bodySkill3);
        
        bodySkill4 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        bodySkill4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[17] = (Integer)bodySkill4.getSelectedIndex();
                }
            });
        bodyPanel.add(bodySkill4);
    }
    
    private void setupBreathArea()
    {
        breathPanel = new JPanel(new GridLayout(4,1,1,1));
        breathPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Breath Skills", TitledBorder.CENTER, TitledBorder.CENTER));
        thirdBox.add(breathPanel);
        
        breathSkill1 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        breathSkill1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[18] = (Integer)breathSkill1.getSelectedIndex();
                }
            });
        breathPanel.add(breathSkill1);
        
        breathSkill2 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        breathSkill2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[19] = (Integer)breathSkill2.getSelectedIndex();
                }
            });
        breathPanel.add(breathSkill2);
        
        breathSkill3 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        breathSkill3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[20] = (Integer)breathSkill3.getSelectedIndex();
                }
            });
        breathPanel.add(breathSkill3);
        
        breathSkill4 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        breathSkill4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[21] = (Integer)breathSkill4.getSelectedIndex();
                }
            });
        breathPanel.add(breathSkill4);
    }
    
    private void setupMagicArea()
    {
        magicPanel = new JPanel(new GridLayout(4,1,1,1));
        magicPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Magic Skills", TitledBorder.CENTER, TitledBorder.CENTER));
        thirdBox.add(magicPanel);
        
        magicSkill1 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        magicSkill1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[22] = (Integer)magicSkill1.getSelectedIndex();
                }
            });
        magicPanel.add(magicSkill1);
        
        magicSkill2 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        magicSkill2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[23] = (Integer)magicSkill2.getSelectedIndex();
                }
            });
        magicPanel.add(magicSkill2);
        
        magicSkill3 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        magicSkill3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[24] = (Integer)magicSkill3.getSelectedIndex();
                }
            });
        magicPanel.add(magicSkill3);
        
        magicSkill4 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        magicSkill4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[25] = (Integer)magicSkill4.getSelectedIndex();
                }
            });
        magicPanel.add(magicSkill4);
    }

    private void initializeMonsterForms2()
    {
        requiredHP.setSelectedItem(currentEntryArray[0]);
        monsterForm.setSelectedItem(DataHandler.ARRAY_STRING_ENEMIES[currentEntryArray[1]]);
        headSkill1.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[2]]);
        headSkill2.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[3]]);
        headSkill3.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[4]]);
        headSkill4.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[5]]);
        armSkill1.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[6]]);
        armSkill2.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[7]]);
        armSkill3.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[8]]);
        armSkill4.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[9]]);
        legSkill1.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[10]]);
        legSkill2.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[11]]);
        legSkill3.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[12]]);
        legSkill4.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[13]]);
        bodySkill1.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[14]]);
        bodySkill2.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[15]]);
        bodySkill3.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[16]]);
        bodySkill4.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[17]]);
        breathSkill1.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[18]]);
        breathSkill2.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[19]]);
        breathSkill3.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[20]]);
        breathSkill4.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[21]]);
        magicSkill1.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[22]]);
        magicSkill2.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[23]]);
        magicSkill3.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[24]]);
        magicSkill4.setSelectedItem(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE[currentEntryArray[25]]);
    }
}
