import java.awt.*;
import java.awt.BorderLayout;
import java.awt.event.*;
import java.io.*;
import java.lang.String.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.BoxLayout;
import javax.swing.event.*;
import java.text.*;
import javax.swing.border.*;
import java.util.HashMap;
/**
 * Class to handle editing mystic absorb data for saga frontier.
 * SnakeOil (c) 2015 - 2019
 * https://bitbucket.org/MysticLord/
 */public class MysticAbsorbs extends JFrame
{
    //local references to static fields
    private HashMap<String, Integer[]> mysticDataStruct;
    private Integer[] currentEntryArray;

    //misc class fields
    String enemyName;

    //Container area
    private JScrollPane mysticScrollPane;
    private Box FrameBox;
    private JPanel BorderPanel;
    private JPanel FlowPanel;
    private Box SkillBox;

    //enemy selector area
    private Box enemySelectorBox;
    private JList<String> enemySelector;
    private JScrollPane enemySelectorScrollPane;

    //stat boost area
    private Box statBox;

    private Box statAmountBox;
    private JLabel statAmountLabel;
    private JComboBox<Integer> statAmount;

    private JPanel statCheckBoxPanel;
    private JCheckBox strButton;
    private JCheckBox quiButton;
    private JCheckBox intButton;
    private JCheckBox wilButton;
    private JCheckBox psyButton;
    private JCheckBox vitButton;
    private JCheckBox chaButton;
    private JCheckBox HPButton;

    
    //skill 1 area
    private Box skill1Box;

    private JComboBox<String> skill1;

    private JPanel byte3Panel;
    private JCheckBox byte3Bit1;
    private JCheckBox byte3Bit2;
    private JCheckBox byte3Bit3;
    private JCheckBox byte3Bit4;
    private JCheckBox byte3Bit5;
    private JCheckBox byte3Bit6;
    private JCheckBox byte3Bit7;

    //skill 2 area
    private Box skill2Box;

    private JComboBox<String> skill2;

    private JPanel byte5Panel;
    private JCheckBox byte5Bit1;
    private JCheckBox byte5Bit2;
    private JCheckBox byte5Bit3;
    private JCheckBox byte5Bit4;
    private JCheckBox byte5Bit5;
    private JCheckBox byte5Bit6;
    private JCheckBox byte5Bit7;

    //skill 3 area
    private Box skill3Box;

    private JComboBox<String> skill3;

    private JPanel byte7Panel;
    private JCheckBox byte7Bit1;
    private JCheckBox byte7Bit2;
    private JCheckBox byte7Bit3;
    private JCheckBox byte7Bit4;
    private JCheckBox byte7Bit5;
    private JCheckBox byte7Bit6;
    private JCheckBox byte7Bit7;


    /**
     * Constructor for objects of class MysticAbsorbs
     */
    public MysticAbsorbs()
    {
        super("Mystic Absorbs");

        for (FixedSizeDataTable dataTable : DataHandler.dataTables)
        {
            if (dataTable.getTableName().equals("mysticAbsorbs"))
            {
                //find local reference so you don't need to search every time
                //  it is modified or accessed.
                mysticDataStruct = dataTable.data;
                break;//quit when you find it - no reason to keep searching.
            }
        }

        setupContainers();
        setupSelector();
        setupStatBoosts();

        FrameBox.add(SkillBox);
        
        setupSkill1Area();
        setupSkill2Area();
        setupSkill3Area();

        enemySelector.setSelectedIndex(0);
        enemyName = enemySelector.getSelectedValue();
        currentEntryArray = mysticDataStruct.get(enemyName);
        initializeMysticAbsorbs();
        pack();
        setVisible(true);
    }

    /**
     * Setup top level containers.
     */
    private void setupContainers()
    {
        //essential, do not modify
        BorderPanel = new JPanel(new BorderLayout());
        FlowPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        FrameBox = new Box(BoxLayout.X_AXIS);

        BorderPanel.add(FrameBox, BorderLayout.NORTH);
        FlowPanel.add(BorderPanel);

        mysticScrollPane = new JScrollPane(FlowPanel, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        getContentPane().add(mysticScrollPane);

        //non-essential design stuff
        SkillBox = new Box(BoxLayout.X_AXIS);
    }

    private void setupSelector()
    {
        //enemy selector area
        enemySelectorBox = new Box(BoxLayout.Y_AXIS);//axis shouldn't matter here, but this is top-to-bottom
        enemySelectorBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Selector", TitledBorder.CENTER, TitledBorder.CENTER));
        FrameBox.add(enemySelectorBox);

        enemySelector = new JList<String>(DataHandler.ARRAY_STRING_ENEMIES);
        enemySelector.setVisibleRowCount(12);
        enemySelector.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        enemySelector.addListSelectionListener(new ListSelectionListener()
            {
                public void valueChanged(ListSelectionEvent e)
                {
                    enemyName = (String)enemySelector.getSelectedValue();
                    currentEntryArray = mysticDataStruct.get(enemyName);
                    initializeMysticAbsorbs();
                }
            });

        enemySelectorScrollPane = new JScrollPane (enemySelector, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        enemySelectorBox.add(enemySelectorScrollPane);
    }

    private void setupStatBoosts()
    {
        statBox = new Box(BoxLayout.Y_AXIS);
        statBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Stat Boost", TitledBorder.CENTER, TitledBorder.CENTER));
        FrameBox.add(statBox);

        statAmount = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        statAmount.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (currentEntryArray[1] != (Integer)statAmount.getSelectedItem())
                    {
                        currentEntryArray[1] = (Integer)statAmount.getSelectedItem();
                    }
                }
            });
        statBox.add(statAmount);
        
        statCheckBoxPanel = new JPanel(new GridLayout(8,1));
        statBox.add(statCheckBoxPanel);

        strButton = new JCheckBox("Strength", false);
        quiButton = new JCheckBox("Quickness", false);
        intButton = new JCheckBox("Intelligence", false);
        wilButton = new JCheckBox("Will", false);
        psyButton = new JCheckBox("Psychic", false);
        vitButton = new JCheckBox("Vitality", false);
        chaButton = new JCheckBox("Charm", false);
        HPButton = new JCheckBox("HP", false);

        statCheckBoxPanel.add(strButton);
        statCheckBoxPanel.add(quiButton);
        statCheckBoxPanel.add(intButton);
        statCheckBoxPanel.add(wilButton);
        statCheckBoxPanel.add(psyButton);
        statCheckBoxPanel.add(vitButton);
        statCheckBoxPanel.add(chaButton);
        statCheckBoxPanel.add(HPButton);

        strButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (strButton.isSelected())
                    {
                        currentEntryArray[0] |= 0x01;
                    }
                    else
                    {
                        currentEntryArray[0] &= 0xFE;
                    }
                }
            });
        quiButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (quiButton.isSelected())
                    {
                        currentEntryArray[0] |= 0x02;
                    }
                    else
                    {
                        currentEntryArray[0] &= 0xFD;
                    }
                }
            });
        intButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (intButton.isSelected())
                    {
                        currentEntryArray[0] |= 0x04;
                    }
                    else
                    {
                        currentEntryArray[0] &= 0xFB;
                    }
                }
            });
        wilButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (wilButton.isSelected())
                    {
                        currentEntryArray[0] |= 0x08;
                    }
                    else
                    {
                        currentEntryArray[0] &= 0xF7;
                    }
                }
            });
        psyButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (psyButton.isSelected())
                    {
                        currentEntryArray[0] |= 0x10;
                    }
                    else
                    {
                        currentEntryArray[0] &= 0xEF;
                    }
                }
            });
        vitButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (vitButton.isSelected())
                    {
                        currentEntryArray[0] |= 0x20;
                    }
                    else
                    {
                        currentEntryArray[0] &= 0xDF;
                    }
                }
            });
        chaButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (chaButton.isSelected())
                    {
                        currentEntryArray[0] |= 0x40;
                    }
                    else
                    {
                        currentEntryArray[0] &= 0xBF;
                    }
                }
            });
        HPButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (HPButton.isSelected())
                    {
                        currentEntryArray[0] |= 0x80;
                    }
                    else
                    {
                        currentEntryArray[0] &= 0x7F;
                    }
                }
            });
    }
    
    private void setupSkill1Area()
    {
        //skill 1 area
        skill1Box = new Box(BoxLayout.Y_AXIS);
        skill1Box.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Mystic Sword", TitledBorder.CENTER, TitledBorder.CENTER));
        SkillBox.add(skill1Box);

        skill1 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES);
        skill1Box.add(skill1);

        skill1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (skill1.getSelectedItem().toString().charAt(0) == '1')
                    {
                        //selected skill ID is greater than 0x100, update 0x00??.
                        currentEntryArray[2] = skill1.getSelectedIndex() - 256;
                        if ((Integer)currentEntryArray[3] % 2 != 1)
                        {
                            //0x??00 hasn't been updated, do so now.
                            currentEntryArray[3] += 1;
                        }
                    }
                    else
                    {
                        //selected skill ID is less than 0x100, update 0x00??.
                        currentEntryArray[2] = skill1.getSelectedIndex();
                        if (currentEntryArray[3] % 2 == 1)
                        {
                            //0x??00 hasn't been updated, do so now.
                            currentEntryArray[3] -= 1;
                        }
                    }
                    //note: this whole section could be easily replaced with some bit shifts
                    //set mysticDataStruct[2] = to selectedValue & 0xFF
                    //set mysticDataStruct[3] = to selectedValue > 8... but need to logical something tho
                }
            });
        skill1Box.add(skill1);
        
        byte3Panel = new JPanel(new GridLayout(8, 1));
        skill1Box.add(byte3Panel);

        byte3Bit1 = new JCheckBox("Byte 3, bit 1");
        byte3Bit2 = new JCheckBox("Byte 3, bit 2");
        byte3Bit3 = new JCheckBox("Byte 3, bit 3");
        byte3Bit4 = new JCheckBox("Byte 3, bit 4");
        byte3Bit5 = new JCheckBox("Byte 3, bit 5");
        byte3Bit6 = new JCheckBox("Byte 3, bit 6");
        byte3Bit7 = new JCheckBox("Byte 3, bit 7");

        byte3Panel.add(byte3Bit1);
        byte3Panel.add(byte3Bit2);
        byte3Panel.add(byte3Bit3);
        byte3Panel.add(byte3Bit4);
        byte3Panel.add(byte3Bit5);
        byte3Panel.add(byte3Bit6);
        byte3Panel.add(byte3Bit7);

        byte3Bit1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte3Bit1.isSelected())
                    {
                        currentEntryArray[3] |= 0x02;
                    }
                    else
                    {
                        currentEntryArray[3] &= 0xFD;
                    }
                }
            });
        byte3Bit2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte3Bit2.isSelected())
                    {
                        currentEntryArray[3] |= 0x04;
                    }
                    else
                    {
                        currentEntryArray[3] &= 0xFB;
                    }
                }
            });
        byte3Bit3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte3Bit3.isSelected())
                    {
                        currentEntryArray[3] |= 0x08;
                    }
                    else
                    {
                        currentEntryArray[3] &= 0xF7;
                    }
                }
            });
        byte3Bit4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte3Bit4.isSelected())
                    {
                        currentEntryArray[3] |= 0x10;
                    }
                    else
                    {
                        currentEntryArray[3] &= 0xEF;
                    }
                }
            });
        byte3Bit5.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte3Bit5.isSelected())
                    {
                        currentEntryArray[3] |= 0x20;
                    }
                    else
                    {
                        currentEntryArray[3] &= 0xDF;
                    }
                }
            });
        byte3Bit6.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte3Bit6.isSelected())
                    {
                        currentEntryArray[3] |= 0x40;
                    }
                    else
                    {
                        currentEntryArray[3] &= 0xBF;
                    }
                }
            });
        byte3Bit7.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte3Bit7.isSelected())
                    {
                        currentEntryArray[3] |= 0x80;
                    }
                    else
                    {
                        currentEntryArray[3] &= 0x7F;
                    }
                }
            });
    }

    private void setupSkill2Area()
    {
        //skill 2 area
        skill2Box = new Box(BoxLayout.Y_AXIS);
        skill2Box.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Mystic Gloves", TitledBorder.CENTER, TitledBorder.CENTER));
        SkillBox.add(skill2Box);

        skill2 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES);
        skill2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (skill2.getSelectedItem().toString().charAt(0) == '1')
                    {
                        currentEntryArray[4] = skill2.getSelectedIndex() - 256;
                        if (currentEntryArray[5] % 2 != 1)
                        {
                            currentEntryArray[5] += 1;
                        }
                    }
                    else
                    {
                        currentEntryArray[4] = skill2.getSelectedIndex();
                        if (currentEntryArray[5] % 2 == 1)
                        {
                            currentEntryArray[5] -= 1;
                        }
                    }
                }
            });
        skill2Box.add(skill2);

        byte5Panel = new JPanel(new GridLayout(8, 1));
        skill2Box.add(byte5Panel);

        byte5Bit1 = new JCheckBox("Byte 5, bit 1");
        byte5Bit2 = new JCheckBox("Byte 5, bit 2");
        byte5Bit3 = new JCheckBox("Byte 5, bit 3");
        byte5Bit4 = new JCheckBox("Byte 5, bit 4");
        byte5Bit5 = new JCheckBox("Byte 5, bit 5");
        byte5Bit6 = new JCheckBox("Byte 5, bit 6");
        byte5Bit7 = new JCheckBox("Byte 5, bit 7");

        byte5Panel.add(byte5Bit1);
        byte5Panel.add(byte5Bit2);
        byte5Panel.add(byte5Bit3);
        byte5Panel.add(byte5Bit4);
        byte5Panel.add(byte5Bit5);
        byte5Panel.add(byte5Bit6);
        byte5Panel.add(byte5Bit7);
        
        byte5Bit1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte5Bit1.isSelected())
                    {
                        currentEntryArray[5] |= 0x02;
                    }
                    else
                    {
                        currentEntryArray[5] &= 0xFD;
                    }
                }
            });
        byte5Bit2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte5Bit2.isSelected())
                    {
                        currentEntryArray[5] |= 0x04;
                    }
                    else
                    {
                        currentEntryArray[5] &= 0xFB;
                    }
                }
            });
        byte5Bit3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte5Bit3.isSelected())
                    {
                        currentEntryArray[5] |= 0x08;
                    }
                    else
                    {
                        currentEntryArray[5] &= 0xF7;
                    }
                }
            });
        byte5Bit4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte5Bit4.isSelected())
                    {
                        currentEntryArray[5] |= 0x10;
                    }
                    else
                    {
                        currentEntryArray[5] &= 0xEF;
                    }
                }
            });
        byte5Bit5.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte5Bit5.isSelected())
                    {
                        currentEntryArray[5] |= 0x20;
                    }
                    else
                    {
                        currentEntryArray[5] &= 0xDF;
                    }
                }
            });
        byte5Bit6.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte5Bit6.isSelected())
                    {
                        currentEntryArray[5] |= 0x40;
                    }
                    else
                    {
                        currentEntryArray[5] &= 0xBF;
                    }
                }
            });
        byte5Bit7.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte5Bit7.isSelected())
                    {
                        currentEntryArray[5] |= 0x80;
                    }
                    else
                    {
                        currentEntryArray[5] &= 0x7F;
                    }
                }
            });
    }

    private void setupSkill3Area()
    {
        //skill 3 area
        skill3Box = new Box(BoxLayout.Y_AXIS);
        skill3Box.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Mystic Boots", TitledBorder.CENTER, TitledBorder.CENTER));
        SkillBox.add(skill3Box);

        skill3 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES);
        skill3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (skill3.getSelectedItem().toString().charAt(0) == '1')
                    {
                        currentEntryArray[6] = skill3.getSelectedIndex() - 256;
                        if (currentEntryArray[7] % 2 != 1)
                        {
                            currentEntryArray[7] += 1;
                        }
                    }
                    else
                    {
                        currentEntryArray[6] = skill3.getSelectedIndex();
                        if (currentEntryArray[7] % 2 == 1)
                        {
                            currentEntryArray[7] -= 1;
                        }
                    }
                }
            });
        skill3Box.add(skill3);

        byte7Panel = new JPanel(new GridLayout(8, 1));
        skill3Box.add(byte7Panel);

        byte7Bit1 = new JCheckBox("Byte 7, bit 1");
        byte7Bit2 = new JCheckBox("Byte 7, bit 2");
        byte7Bit3 = new JCheckBox("Byte 7, bit 3");
        byte7Bit4 = new JCheckBox("Byte 7, bit 4");
        byte7Bit5 = new JCheckBox("Byte 7, bit 5");
        byte7Bit6 = new JCheckBox("Byte 7, bit 6");
        byte7Bit7 = new JCheckBox("Byte 7, bit 7");

        byte7Panel.add(byte7Bit1);
        byte7Panel.add(byte7Bit2);
        byte7Panel.add(byte7Bit3);
        byte7Panel.add(byte7Bit4);
        byte7Panel.add(byte7Bit5);
        byte7Panel.add(byte7Bit6);
        byte7Panel.add(byte7Bit7);
        
        byte7Bit1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte7Bit1.isSelected())
                    {
                        currentEntryArray[7] |= 0x02;
                    }
                    else
                    {
                        currentEntryArray[7] &= 0xFD;
                    }
                }
            });
        byte7Bit2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte7Bit2.isSelected())
                    {
                        currentEntryArray[7] |= 0x04;
                    }
                    else
                    {
                        currentEntryArray[7] &= 0xFB;
                    }
                }
            });
        byte7Bit3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte7Bit3.isSelected())
                    {
                        currentEntryArray[7] |= 0x08;
                    }
                    else
                    {
                        currentEntryArray[7] &= 0xF7;
                    }
                }
            });
        byte7Bit4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte7Bit4.isSelected())
                    {
                        currentEntryArray[7] |= 0x10;
                    }
                    else
                    {
                        currentEntryArray[7] &= 0xEF;
                    }
                }
            });
        byte7Bit5.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte7Bit5.isSelected())
                    {
                        currentEntryArray[7] |= 0x20;
                    }
                    else
                    {
                        currentEntryArray[7] &= 0xDF;
                    }
                }
            });
        byte7Bit6.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte7Bit6.isSelected())
                    {
                        currentEntryArray[7] |= 0x40;
                    }
                    else
                    {
                        currentEntryArray[7] &= 0xBF;
                    }
                }
            });
        byte7Bit7.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    if (byte7Bit7.isSelected())
                    {
                        currentEntryArray[7] |= 0x80;
                    }
                    else
                    {
                        currentEntryArray[7] &= 0x7F;
                    }
                }
            });
    }

    private void initializeMysticAbsorbs()
    {
        //stat boosts area
        statAmount.setSelectedItem(currentEntryArray[1]);
        if ((currentEntryArray[0] & 0x01) == 0x01)
        {
            strButton.setSelected(true);
        }
        else
        {
            strButton.setSelected(false);
        }
        if ((currentEntryArray[0] & 0x02) == 0x02)
        {
            quiButton.setSelected(true);
        }
        else
        {
            quiButton.setSelected(false);
        }
        if ((currentEntryArray[0] & 0x04) == 0x04)
        {
            intButton.setSelected(true);
        }
        else
        {
            intButton.setSelected(false);
        }
        if ((currentEntryArray[0] & 0x08) == 0x08)
        {
            wilButton.setSelected(true);
        }
        else
        {
            wilButton.setSelected(false);
        }
        if ((currentEntryArray[0] & 0x10) == 0x10)
        {
            psyButton.setSelected(true);
        }
        else
        {
            psyButton.setSelected(false);
        }
        if ((currentEntryArray[0] & 0x20) == 0x20)
        {
            vitButton.setSelected(true);
        }
        else
        {
            vitButton.setSelected(false);
        }
        if ((currentEntryArray[0] & 0x40) == 0x40)
        {
            chaButton.setSelected(true);
        }
        else
        {
            chaButton.setSelected(false);
        }
        if ((currentEntryArray[0] & 0x80) == 0x80)
        {
            HPButton.setSelected(true);
        }
        else
        {
            HPButton.setSelected(false);
        }

        //skill 1 area
        int tempSkill1Int = currentEntryArray[2] & 0xFF;
        if ((currentEntryArray[3] & 0x01) == 0x01)
        {
            tempSkill1Int |= 0x100;
        }
        skill1.setSelectedIndex(tempSkill1Int);
        if ((currentEntryArray[3] & 0x02) == 0x02)
        {
            byte3Bit1.setSelected(true);
        }
        else
        {
            byte3Bit1.setSelected(false);
        }
        if ((currentEntryArray[3] & 0x04) == 0x04)
        {
            byte3Bit2.setSelected(true);
        }
        else
        {
            byte3Bit2.setSelected(false);
        }
        if ((currentEntryArray[3] & 0x08) == 0x08)
        {
            byte3Bit3.setSelected(true);
        }
        else
        {
            byte3Bit3.setSelected(false);
        }
        if ((currentEntryArray[3] & 0x10) == 0x10)
        {
            byte3Bit4.setSelected(true);
        }
        else
        {
            byte3Bit4.setSelected(false);
        }
        if ((currentEntryArray[3] & 0x20) == 0x20)
        {
            byte3Bit5.setSelected(true);
        }
        else
        {
            byte3Bit5.setSelected(false);
        }
        if ((currentEntryArray[3] & 0x40) == 0x40)
        {
            byte3Bit6.setSelected(true);
        }
        else
        {
            byte3Bit6.setSelected(false);
        }
        if ((currentEntryArray[3] & 0x80) == 0x80)
        {
            byte3Bit7.setSelected(true);
        }
        else
        {
            byte3Bit7.setSelected(false);
        }

        //skill 2 area
        int tempSkill2Int = currentEntryArray[4] & 0xFF;
        if ((currentEntryArray[5] & 0x01) == 0x01)
        {
            tempSkill2Int |= 0x100;
        }
        skill2.setSelectedIndex(tempSkill2Int);
        if ((currentEntryArray[5] & 0x02) == 0x02)
        {
            byte5Bit1.setSelected(true);
        }
        else
        {
            byte5Bit1.setSelected(false);
        }
        if ((currentEntryArray[5] & 0x04) == 0x04)
        {
            byte5Bit2.setSelected(true);
        }
        else
        {
            byte5Bit2.setSelected(false);
        }
        if ((currentEntryArray[5] & 0x08) == 0x08)
        {
            byte5Bit3.setSelected(true);
        }
        else
        {
            byte5Bit3.setSelected(false);
        }
        if ((currentEntryArray[5] & 0x10) == 0x10)
        {
            byte5Bit4.setSelected(true);
        }
        else
        {
            byte5Bit4.setSelected(false);
        }
        if ((currentEntryArray[5] & 0x20) == 0x20)
        {
            byte5Bit5.setSelected(true);
        }
        else
        {
            byte5Bit5.setSelected(false);
        }
        if ((currentEntryArray[5] & 0x40) == 0x40)
        {
            byte5Bit6.setSelected(true);
        }
        else
        {
            byte5Bit6.setSelected(false);
        }
        if ((currentEntryArray[5] & 0x80) == 0x80)
        {
            byte5Bit7.setSelected(true);
        }
        else
        {
            byte5Bit7.setSelected(false);
        }

        //skill 3 area
        int tempSkill3Int = currentEntryArray[6] & 0xFF;
        if ((currentEntryArray[7] & 0x01) == 0x01)
        {
            tempSkill3Int |= 0x100;
        }
        skill3.setSelectedIndex(tempSkill3Int);
        if ((currentEntryArray[7] & 0x02) == 0x02)
        {
            byte7Bit1.setSelected(true);
        }
        else
        {
            byte7Bit1.setSelected(false);
        }
        if ((currentEntryArray[7] & 0x04) == 0x04)
        {
            byte7Bit2.setSelected(true);
        }
        else
        {
            byte7Bit2.setSelected(false);
        }
        if ((currentEntryArray[7] & 0x08) == 0x08)
        {
            byte7Bit3.setSelected(true);
        }
        else
        {
            byte7Bit3.setSelected(false);
        }
        if ((currentEntryArray[7] & 0x10) == 0x10)
        {
            byte7Bit4.setSelected(true);
        }
        else
        {
            byte7Bit4.setSelected(false);
        }
        if ((currentEntryArray[7] & 0x20) == 0x20)
        {
            byte7Bit5.setSelected(true);
        }
        else
        {
            byte7Bit5.setSelected(false);
        }
        if ((currentEntryArray[7] & 0x40) == 0x40)
        {
            byte7Bit6.setSelected(true);
        }
        else
        {
            byte7Bit6.setSelected(false);
        }
        if ((currentEntryArray[7] & 0x80) == 0x80)
        {
            byte7Bit7.setSelected(true);
        }
        else
        {
            byte7Bit7.setSelected(false);
        }
    }
}