import java.awt.*;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.io.*;
import java.lang.String.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.BoxLayout;
import javax.swing.event.*;
import java.text.*;
import javax.swing.border.*;
import java.util.HashMap;
/**
 * Class to handle editing monster transformation data (table 1) for saga frontier.
 * SnakeOil (c) 2015 - 2019
 * https://bitbucket.org/MysticLord/
 */
public class MonsterForms1 extends JFrame
{
    //local references to static fields
    private HashMap<String, Integer[]> monsterDataStruct;
    private Integer[] currentEntryArray;

    //misc class fields
    private String currentEntryName;

    //container area
    private JScrollPane monsterScrollPane;
    private Box FrameBox;
    private JPanel BorderPanel;
    private JPanel FlowPanel;

    //form selector area
    private Box formSelectorBox;
    private JList<String> formSelector;
    private JScrollPane formSelectorScrollPane;

    //other area
    private Box dataBox;
    private JComboBox<String> form;
    private JComboBox<String> tech1;
    private JComboBox<String> tech2;
    private JComboBox<String> tech3;
    private JComboBox<String> tech4;
    private JComboBox<String> tech5;
    private JComboBox<String> tech6;
    private JComboBox<String> tech7;
    private JComboBox<String> tech8;
    
    /**
     * Constructor for objects of class MonsterForms1
     */
    public MonsterForms1()
    {
        super("Monster Forms, Table 1");

        for (FixedSizeDataTable dataTable : DataHandler.dataTables)
        {
            if (dataTable.getTableName().equals("monsterForms1"))
            {//never entering this point..., check data handler
                //find and link local reference
                monsterDataStruct = dataTable.data;
                break;//quit when you find it - no reason to keep searching.
            }
        }

        setupContainers();
        setupSelector();
        setupOtherArea();

        //formSelector.setSelectedIndex(0);
        //these next 3 lines may be redundant due to action listener in formSelector
        //currentEntryName = "0";
        currentEntryArray = monsterDataStruct.get("0");
        //possibly an issue with using number-strings?
        //change to Slot 0x00 to Slot 0x0f
        
        initializeMonsterForms1();
        pack();
        setVisible(true);
    }

    /**
     * Setup top level containers.
     */
    private void setupContainers()
    {
        //essential, do not modify
        BorderPanel = new JPanel(new BorderLayout());
        FlowPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        FrameBox = new Box(BoxLayout.X_AXIS);

        BorderPanel.add(FrameBox, BorderLayout.NORTH);
        FlowPanel.add(BorderPanel);

        monsterScrollPane = new JScrollPane(FlowPanel, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        getContentPane().add(monsterScrollPane);
    }

    private void setupSelector()
    {
        //form selector area
        formSelectorBox = new Box(BoxLayout.Y_AXIS);
        formSelectorBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED),"#", TitledBorder.CENTER, TitledBorder.CENTER));
        FrameBox.add(formSelectorBox);

        formSelector = new JList<String>(DataHandler.ARRAY_STRING_NIBBLE);
        formSelector.setVisibleRowCount(16);
        formSelector.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        formSelector.addListSelectionListener(new ListSelectionListener()
            {
                public void valueChanged(ListSelectionEvent e)
                {
                    currentEntryName = (String)formSelector.getSelectedValue();
                    currentEntryArray = monsterDataStruct.get(currentEntryName);
                    initializeMonsterForms1();
                }
            });
        formSelectorBox.add(formSelector);
    }

    private void setupOtherArea()
    {
        dataBox = new Box(BoxLayout.Y_AXIS);
        dataBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Form and Skills", TitledBorder.CENTER, TitledBorder.CENTER));
        FrameBox.add(dataBox);
        
        form = new JComboBox<String>(DataHandler.ARRAY_STRING_ENEMIES);
        tech1 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        tech2 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        tech3 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        tech4 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        tech5 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        tech6 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        tech7 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        tech8 = new JComboBox<String>(DataHandler.ARRAY_STRING_ABILITIES_LOW_BYTE);
        
        dataBox.add(form);
        dataBox.add(tech1);
        dataBox.add(tech2);
        dataBox.add(tech3);
        dataBox.add(tech4);
        dataBox.add(tech5);
        dataBox.add(tech6);
        dataBox.add(tech7);
        dataBox.add(tech8);

        form.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[0] = form.getSelectedIndex();
                }
            });
        tech1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[1] = tech1.getSelectedIndex();
                }
            });
        tech2.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[2] = tech2.getSelectedIndex();
                }
            });
        tech3.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[3] = tech3.getSelectedIndex();
                }
            });
        tech4.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[4] = tech4.getSelectedIndex();
                }
            });
        tech5.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[5] = tech5.getSelectedIndex();
                }
            });
        tech6.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[6] = tech6.getSelectedIndex();
                }
            });
        tech7.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[7] = tech7.getSelectedIndex();
                }
            });
        tech8.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    currentEntryArray[8] = tech8.getSelectedIndex();
                }
            });
    }

    private void initializeMonsterForms1()
    {
        form.setSelectedIndex(currentEntryArray[0]);
        tech1.setSelectedIndex(currentEntryArray[1]);
        tech2.setSelectedIndex(currentEntryArray[2]);
        tech3.setSelectedIndex(currentEntryArray[3]);
        tech4.setSelectedIndex(currentEntryArray[4]);
        tech5.setSelectedIndex(currentEntryArray[5]);
        tech6.setSelectedIndex(currentEntryArray[6]);
        tech7.setSelectedIndex(currentEntryArray[7]);
        tech8.setSelectedIndex(currentEntryArray[8]);
    }
}