import java.awt.*;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import java.io.*;
import java.lang.String.*;
import java.lang.String;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.BoxLayout;
import javax.swing.event.*;
import java.text.*;
import javax.swing.border.*;
import java.util.HashMap;
/**
 * Class to handle editing skills for saga frontier.
 * SnakeOil (c) 2015 - 2019
 * https://bitbucket.org/MysticLord/
 */
public class Skills extends JFrame
{
    //local references to static fields
    private HashMap<String, Integer[]> skillsDataStruct;
    private Integer[] currentEntryArray;

    //misc class fields
    private String currentEntryName;
    
    //menu area
    private JMenuBar skillMenuBar;
    private JMenu skillMenu;
    private JMenuItem skillHelp;
    private JFrame skillHelpFrame;
    private Box skillHelpBox;
    private JTextArea skillHelpTextArea;

    //container area
    private JScrollPane skillsScrollPane;
    private Box FrameBox;
    private JPanel BorderPanel;
    private JPanel FlowPanel;
    
    //mid-level containers
    private Box column3Box;
    private Box column4Box;

    //skill selector area
    private Box skillSelectorBox;
    private JScrollPane skillSelectorScrollPane;
    private JList<String> skillSelector;

    //misc area - add two subboxes, change orientation of miscbox to left-right
    private Box miscBox;
    private Box miscBoxLeft;
    private Box miscBoxRight;
    private JLabel accuracyLabel;
    private JComboBox<Integer> accuracy;
    private JLabel basePowerLabel;
    private JComboBox<Integer> basePower;
    private JLabel powerGrowthLabel;
    private JComboBox<Integer> powerGrowth;
    private JLabel additionalEffectLabel;
    private JComboBox<Integer> additionalEffect;
    private JLabel priorityLabel;
    private JComboBox<Integer> priority;
    private JLabel targetingLabel;
    private JComboBox<Integer> targeting;
    private JLabel areaOfEffectLabel;
    private JComboBox<Integer> areaOfEffect;
    private JLabel effectsProcessLabel;
    private JComboBox<Integer> effectsProcess;
    private JLabel ammoUsedLabel;
    private JComboBox<Integer> ammoUsed;

    //unknown area
    private Box unknownBox;
    private JLabel byte0x09Label;
    private JComboBox<Integer> byte0x09;
    private JLabel skillUseLimitsLabel;
    private JComboBox<Integer> skillUseLimits;
    private JLabel byte0x0bLabel;
    private JComboBox<Integer> byte0x0b;
    private JLabel comboSendLinkSkillLabel;
    private JComboBox<Integer> comboSendLinkSkill;
    private JLabel byte0x0eLabel;
    private JComboBox<Integer> byte0x0e;
    private JLabel comboReceiveLabel;
    private JComboBox<Integer> comboReceive;

    //cost area
    private Box costBox;//move to separate method
    private JRadioButton wpSelector;
    private JRadioButton jpSelector;
    private ButtonGroup costGroup;
    private JCheckBox useAll;
    private JComboBox<Integer> skillCost;

    //attributes area
    private JPanel attributesPanel;
    private JButton attributeNotesButton;
    private JLabel attrLabel;
    private JCheckBox attrAntiAir;
    private JCheckBox attrBit1;
    private JCheckBox attrContact;
    private JCheckBox attrLiquid;
    private JCheckBox attrNonContact;
    private JCheckBox attrSonic;
    private JCheckBox attrGaze;
    private JCheckBox attrGround;

    //elements area
    private JPanel elementsPanel;
    private JCheckBox eleStatus;
    private JCheckBox eleForce;
    private JCheckBox eleElectric;
    private JCheckBox eleCold;
    private JCheckBox eleHeat;
    private JCheckBox eleBlunt;
    private JCheckBox elePierce;
    private JCheckBox eleSlash;

    /**
     * Constructor for objects of class Skills
     */
    public Skills()
    {
        super("Skills");

        for (FixedSizeDataTable dataTable : DataHandler.dataTables)
        {
            if (dataTable.getTableName().equals("skills1"))
            {
                //find and link local reference
                skillsDataStruct = dataTable.data;
                break;//quit when you find it - no reason to keep searching.
            }
        }

        setupMenu();
        setupContainers();
        setupSelector();
        setupMiscArea();
        setupMiddleContainers();
        setupUnknowns();
        setupCostArea();
        setupAttributes();
        setupElements();

        pack();
        skillSelector.setSelectedIndex(0);
        currentEntryName = (String)skillSelector.getSelectedValue();
        currentEntryArray = skillsDataStruct.get(currentEntryName);
        initializeSkills1();
        setVisible(true);
    }

    private void setupMenu()
    {
        skillMenuBar = new JMenuBar();
        setJMenuBar(skillMenuBar);
        
        skillMenu = new JMenu("Misc");
        skillMenuBar.add(skillMenu);
        
        skillHelp = new JMenuItem("Help");
        skillHelp.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    skillHelpFrame = new JFrame("Help");
                    
                    skillHelpBox = new Box(BoxLayout.Y_AXIS);
                    skillHelpFrame.add(skillHelpBox);
                    
                    String helpStr = new String(
                    "For detailed explanations, please see the key at the bottom of this webpage: \n" + 
                    "https://bitbucket.org/MysticLord/waza/src/default/ability_data_1.txt\n") +
                    "\n" +
                    "Base 16 to Base 10\n" +
                    "0x01 :   1\n" +
                    "0x02 :   2\n" +
                    "0x04 :   4\n" +
                    "0x08 :   8\n" +
                    "0x10 :  16\n" +
                    "0x20 :  32\n" +
                    "0x40 :  64\n" +
                    "0x80 : 128\n";
                    skillHelpTextArea = new JTextArea(helpStr);
                    skillHelpTextArea.setEditable(false);
                    skillHelpBox.add(skillHelpTextArea);
                    
                    skillHelpFrame.pack();
                    skillHelpFrame.setVisible(true);
                }
            });
        skillMenu.add(skillHelp);
    }
    
    /**
     * Setup top level containers.
     */
    private void setupContainers()
    {
        //essential, do not modify
        BorderPanel = new JPanel(new BorderLayout());
        FlowPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
        FrameBox = new Box(BoxLayout.X_AXIS);

        BorderPanel.add(FrameBox, BorderLayout.NORTH);
        FlowPanel.add(BorderPanel);

        skillsScrollPane = new JScrollPane(FlowPanel, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        getContentPane().add(skillsScrollPane);
    }
    
    private void setupSelector()
    {
        //form selector area
        skillSelectorBox = new Box(BoxLayout.Y_AXIS);
        skillSelectorBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED),"Selector", TitledBorder.CENTER, TitledBorder.CENTER));
        FrameBox.add(skillSelectorBox);

        skillSelector = new JList<String>(DataHandler.ARRAY_STRING_ABILITIES);
        skillSelector.setVisibleRowCount(16);
        skillSelector.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        skillSelector.addListSelectionListener(new ListSelectionListener()
            {
                public void valueChanged(ListSelectionEvent e)
                {
                    currentEntryName = (String)skillSelector.getSelectedValue();
                    currentEntryArray = skillsDataStruct.get(currentEntryName);
                    initializeSkills1();
                }
            });
        //skillSelector.setSelectedValue();
        skillSelectorBox.add(skillSelector);

        skillSelectorScrollPane = new JScrollPane(skillSelector, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        skillSelectorBox.add(skillSelectorScrollPane);
    }

    private void setupMiscArea()
    {
        miscBox = new Box(BoxLayout.X_AXIS);
        miscBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Misc", TitledBorder.CENTER, TitledBorder.CENTER));
        FrameBox.add(miscBox);

        JPanel gridPanelLeft = new JPanel(new GridLayout(9,1,1,1));
        miscBox.add(gridPanelLeft);

        JPanel gridPanelRight = new JPanel(new GridLayout(9,1,1,1));
        miscBox.add(gridPanelRight);

        accuracyLabel = new JLabel("Accuracy");
        gridPanelLeft.add(accuracyLabel);

        accuracy = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        accuracy.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[0] = (Integer)accuracy.getSelectedItem();
                }
            });
        gridPanelRight.add(accuracy);

        basePowerLabel = new JLabel("Base Power");
        gridPanelLeft.add(basePowerLabel);

        basePower = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        basePower.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[1] = (Integer)basePower.getSelectedItem();
                }
            });
        gridPanelRight.add(basePower);

        powerGrowthLabel = new JLabel("Power Growth");
        gridPanelLeft.add(powerGrowthLabel);

        powerGrowth = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        powerGrowth.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[2] = (Integer)powerGrowth.getSelectedItem();
                }
            });
        gridPanelRight.add(powerGrowth);

        additionalEffectLabel = new JLabel("Additional Effect");
        gridPanelLeft.add(additionalEffectLabel);

        additionalEffect = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        additionalEffect.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[3] = (Integer)additionalEffect.getSelectedItem();
                }
            });
        gridPanelRight.add(additionalEffect);

        priorityLabel = new JLabel("Priority");
        gridPanelLeft.add(priorityLabel);

        priority = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        priority.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[4] = (Integer)priority.getSelectedItem();
                }
            });
        gridPanelRight.add(priority);

        targetingLabel = new JLabel("Targeting");
        gridPanelLeft.add(targetingLabel);

        targeting = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        targeting.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[7] = (Integer)targeting.getSelectedItem();
                }
            });
        gridPanelRight.add(targeting);

        areaOfEffectLabel = new JLabel("Area of Effect");
        gridPanelLeft.add(areaOfEffectLabel);

        areaOfEffect = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        areaOfEffect.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[8] = (Integer)areaOfEffect.getSelectedItem();
                }
            });
        gridPanelRight.add(areaOfEffect);

        effectsProcessLabel = new JLabel("Effects Process");
        gridPanelLeft.add(effectsProcessLabel);

        effectsProcess = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        effectsProcess.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[12] = (Integer)effectsProcess.getSelectedItem();
                }
            });
        gridPanelRight.add(effectsProcess);

        ammoUsedLabel = new JLabel("Ammo Used");
        gridPanelLeft.add(ammoUsedLabel);

        ammoUsed = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        ammoUsed.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[17] = (Integer)ammoUsed.getSelectedItem();
                }
            });
        gridPanelRight.add(ammoUsed);

    }

    private void setupMiddleContainers()
    {
        column3Box = new Box(BoxLayout.Y_AXIS);
        FrameBox.add(column3Box);
        
        column4Box = new Box(BoxLayout.Y_AXIS);
        FrameBox.add(column4Box);
    }

    private void setupUnknowns()
    {
        unknownBox = new Box(BoxLayout.X_AXIS);
        unknownBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Other", TitledBorder.CENTER, TitledBorder.CENTER));
        column3Box.add(unknownBox);

        JPanel gridPanelLeft = new JPanel(new GridLayout(6,1,1,1));
        unknownBox.add(gridPanelLeft);

        JPanel gridPanelRight = new JPanel(new GridLayout(6,1,1,1));
        unknownBox.add(gridPanelRight);

        byte0x09Label = new JLabel("Byte 0x09");
        gridPanelLeft.add(byte0x09Label);

        byte0x09 = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        byte0x09.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[9] = (Integer)byte0x09.getSelectedItem();
                }
            });
        gridPanelRight.add(byte0x09);

        skillUseLimitsLabel = new JLabel("Skill Use Limits");
        gridPanelLeft.add(skillUseLimitsLabel);

        skillUseLimits = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        skillUseLimits.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[10] = (Integer)skillUseLimits.getSelectedItem();
                }
            });
        gridPanelRight.add(skillUseLimits);

        byte0x0bLabel = new JLabel("Byte 0x0b");
        gridPanelLeft.add(byte0x0bLabel);

        byte0x0b = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        byte0x0b.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[11] = (Integer)byte0x0b.getSelectedItem();
                }
            });
        gridPanelRight.add(byte0x0b);

        comboSendLinkSkillLabel = new JLabel("Combo Send");
        gridPanelLeft.add(comboSendLinkSkillLabel);

        comboSendLinkSkill = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        comboSendLinkSkill.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[13] = (Integer)comboSendLinkSkill.getSelectedItem();
                }
            });
        gridPanelRight.add(comboSendLinkSkill);

        byte0x0eLabel = new JLabel("Byte 0x0e");
        gridPanelLeft.add(byte0x0eLabel);

        byte0x0e = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        byte0x0e.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[14] = (Integer)byte0x0e.getSelectedItem();
                }
            });
        gridPanelRight.add(byte0x0e);

        comboReceiveLabel = new JLabel("Combo Receive");
        gridPanelLeft.add(comboReceiveLabel);

        comboReceive = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_BYTE);
        comboReceive.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[15] = (Integer)comboReceive.getSelectedItem();
                }
            });
        gridPanelRight.add(comboReceive);

    }

    private void setupCostArea()
    {
        costBox = new Box(BoxLayout.Y_AXIS);
        costBox.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Cost", TitledBorder.CENTER, TitledBorder.CENTER));
        column3Box.add(costBox);
        
        JPanel costPanel = new JPanel(new GridLayout(2,2,1,1));
        costBox.add(costPanel);

        wpSelector = new JRadioButton("WP?");
        wpSelector.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    if ((wpSelector.isSelected()) && ((skillsDataStruct.get(skillName)[16] & 0x40) == 0x40))
                    {
                        skillsDataStruct.get(skillName)[16] -= 0x40;
                    }
                }
            });
        costPanel.add(wpSelector);

        jpSelector = new JRadioButton("JP?");
        jpSelector.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    if ((jpSelector.isSelected()) && ((skillsDataStruct.get(skillName)[16] & 0x40) != 0x40))
                    {
                        skillsDataStruct.get(skillName)[16] |= 0x40;
                    }
                }
            });
        costPanel.add(jpSelector);

        costGroup = new ButtonGroup();
        costGroup.add(wpSelector);
        costGroup.add(jpSelector);

        useAll = new JCheckBox("Use all?");
        useAll.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    if (useAll.isSelected())
                    {
                        skillsDataStruct.get(skillName)[16] |= 0x80;
                    }
                    else
                    {
                        if ((skillsDataStruct.get(skillName)[16] & 0x80) == 0x80)
                        {
                            skillsDataStruct.get(skillName)[16] -= 0x80;
                        }
                    }
                }
            });
        costPanel.add(useAll);

        skillCost = new JComboBox<Integer>(DataHandler.ARRAY_UNSIGNED_QUARTER_BYTE);
        skillCost.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[16] |= (Integer)skillCost.getSelectedItem();
                }
            });
        costPanel.add(skillCost);
    }

    private void setupAttributes()
    {
        attributesPanel = new JPanel(new GridLayout(4, 2));
        attributesPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Attributes", TitledBorder.CENTER, TitledBorder.CENTER));
        column4Box.add(attributesPanel);

        attrAntiAir = new JCheckBox("Anti-Air");
        attrAntiAir.setToolTipText("Weakness to Anti-Air attacks");
        attrBit1 = new JCheckBox("?");
        attrBit1.setToolTipText("Byte 0x05, bit 1");
        attrContact = new JCheckBox("Contact");
        attrContact.setToolTipText("Not sure how Contact works");
        attrLiquid = new JCheckBox("Liquid");
        attrLiquid.setToolTipText("Immunity to Liquid attacks");
        attrNonContact = new JCheckBox("!Contact");
        attrNonContact.setToolTipText("Not sure how Non-Contact works; ! is the symbol for \"not\"");
        attrSonic = new JCheckBox("Sonic");
        attrSonic.setToolTipText("Immunity to Sonic attacks");
        attrGaze = new JCheckBox("Gaze");
        attrGaze.setToolTipText("Immunity to Gaze attacks");
        attrGround = new JCheckBox("Ground");
        attrGround.setToolTipText("Immunity to Ground attacks");

        attrAntiAir.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[5] = eightBitToggles(attrAntiAir, attrBit1, attrContact, attrLiquid, attrNonContact, attrSonic, attrGaze, attrGround);
                }
            });
        attrBit1.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[5] = eightBitToggles(attrAntiAir, attrBit1, attrContact, attrLiquid, attrNonContact, attrSonic, attrGaze, attrGround);
                }
            });
        attrContact.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[5] = eightBitToggles(attrAntiAir, attrBit1, attrContact, attrLiquid, attrNonContact, attrSonic, attrGaze, attrGround);
                }
            });
        attrLiquid.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[5] = eightBitToggles(attrAntiAir, attrBit1, attrContact, attrLiquid, attrNonContact, attrSonic, attrGaze, attrGround);
                }
            });
        attrNonContact.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[5] = eightBitToggles(attrAntiAir, attrBit1, attrContact, attrLiquid, attrNonContact, attrSonic, attrGaze, attrGround);
                }
            });
        attrSonic.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[5] = eightBitToggles(attrAntiAir, attrBit1, attrContact, attrLiquid, attrNonContact, attrSonic, attrGaze, attrGround);
                }
            });
        attrGaze.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[5] = eightBitToggles(attrAntiAir, attrBit1, attrContact, attrLiquid, attrNonContact, attrSonic, attrGaze, attrGround);
                }
            });
        attrGround.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = (String)skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[5] = eightBitToggles(attrAntiAir, attrBit1, attrContact, attrLiquid, attrNonContact, attrSonic, attrGaze, attrGround);
                }
            });

        attributesPanel.add(attrAntiAir);
        attributesPanel.add(attrBit1);
        attributesPanel.add(attrContact);
        attributesPanel.add(attrLiquid);
        attributesPanel.add(attrNonContact);
        attributesPanel.add(attrSonic);
        attributesPanel.add(attrGaze);
        attributesPanel.add(attrGround);
    }

    private void setupElements()
    {
        elementsPanel = new JPanel(new GridLayout(4,2));
        elementsPanel.setBorder(new TitledBorder(new BevelBorder(BevelBorder.RAISED), "Elements", TitledBorder.CENTER, TitledBorder.CENTER));
        column4Box.add(elementsPanel);

        eleStatus = new JCheckBox("Status");
        eleForce = new JCheckBox("Force");
        eleElectric = new JCheckBox("Electric");
        eleCold = new JCheckBox("Cold");
        eleHeat = new JCheckBox("Heat");
        eleBlunt = new JCheckBox("Blunt");
        elePierce = new JCheckBox("Pierce");
        eleSlash = new JCheckBox("Slash");

        elementsPanel.add(eleStatus);
        elementsPanel.add(eleForce);
        elementsPanel.add(eleElectric);
        elementsPanel.add(eleCold);
        elementsPanel.add(eleHeat);
        elementsPanel.add(eleBlunt);
        elementsPanel.add(elePierce);
        elementsPanel.add(eleSlash);

        eleStatus.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[6] = eightBitToggles(eleStatus, eleForce, eleElectric, eleCold, eleHeat, eleBlunt, elePierce, eleSlash);
                }
            });
        eleForce.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[6] = eightBitToggles(eleStatus, eleForce, eleElectric, eleCold, eleHeat, eleBlunt, elePierce, eleSlash);
                }
            });
        eleElectric.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[6] = eightBitToggles(eleStatus, eleForce, eleElectric, eleCold, eleHeat, eleBlunt, elePierce, eleSlash);
                }
            });
        eleCold.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[6] = eightBitToggles(eleStatus, eleForce, eleElectric, eleCold, eleHeat, eleBlunt, elePierce, eleSlash);
                }
            });
        eleHeat.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[6] = eightBitToggles(eleStatus, eleForce, eleElectric, eleCold, eleHeat, eleBlunt, elePierce, eleSlash);
                }
            });
        eleBlunt.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[6] = eightBitToggles(eleStatus, eleForce, eleElectric, eleCold, eleHeat, eleBlunt, elePierce, eleSlash);
                }
            });
        elePierce.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[6] = eightBitToggles(eleStatus, eleForce, eleElectric, eleCold, eleHeat, eleBlunt, elePierce, eleSlash);
                }
            });
        eleSlash.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent evt)
                {
                    String skillName = skillSelector.getSelectedValue();
                    skillsDataStruct.get(skillName)[6] = eightBitToggles(eleStatus, eleForce, eleElectric, eleCold, eleHeat, eleBlunt, elePierce, eleSlash);
                }
            });
    }

    private void initializeSkills1()
    {
        //set misc area
        accuracy.setSelectedItem(currentEntryArray[0]);
        basePower.setSelectedItem(currentEntryArray[1]);
        powerGrowth.setSelectedItem(currentEntryArray[2]);
        additionalEffect.setSelectedItem(currentEntryArray[3]);
        priority.setSelectedItem(currentEntryArray[4]);
        targeting.setSelectedItem(currentEntryArray[7]);
        areaOfEffect.setSelectedItem(currentEntryArray[8]);
        byte0x09.setSelectedItem(currentEntryArray[9]);
        skillUseLimits.setSelectedItem(currentEntryArray[10]);
        byte0x0b.setSelectedItem(currentEntryArray[11]);
        effectsProcess.setSelectedItem(currentEntryArray[12]);
        comboSendLinkSkill.setSelectedItem(currentEntryArray[13]);
        
        byte0x0e.setSelectedItem(currentEntryArray[14]);
        comboReceive.setSelectedItem(currentEntryArray[15]);
        
        int costInt = currentEntryArray[16];
        if ((costInt & 0x40) != 0x40)
        {
            wpSelector.setSelected(true);
        }
        else
        {
            jpSelector.setSelected(true);
            costInt -= 0x40;
        }
        if ((costInt & 0x80) == 0x80)
        {
            useAll.setSelected(true);
            costInt -= 0x80;
        }
        else
        {
            useAll.setSelected(false);
        }
        skillCost.setSelectedItem(costInt);
        
        ammoUsed.setSelectedItem(currentEntryArray[17]);
        
        //set attributes - later on replace with a bitwise and check
        boolean[] attrBools = {false, false, false, false, false, false, false, false};
        int attrInt = currentEntryArray[5];
        if (attrInt >= 128)
        {
            attrBools[0] = true;
            attrInt -= 128;
        }
        if (attrInt >= 64)
        {
            attrBools[1] = true;
            attrInt -= 64;
        }
        if (attrInt >= 32)
        {
            attrBools[2] = true;
            attrInt -= 32;
        }
        if (attrInt >= 16)
        {
            attrBools[3] = true;
            attrInt -= 16;
        }
        if (attrInt >= 8)
        {
            attrBools[4] = true;
            attrInt -= 8;
        }
        if (attrInt >= 4)
        {
            attrBools[5] = true;
            attrInt -= 4;
        }
        if (attrInt >= 2)
        {
            attrBools[6] = true;
            attrInt -= 2;
        }
        if (attrInt >= 1)
        {
            attrBools[7] = true;
            attrInt -= 1;
        }
        attrGround.setSelected(attrBools[0]);
        attrGaze.setSelected(attrBools[1]);
        attrSonic.setSelected(attrBools[2]);
        attrNonContact.setSelected(attrBools[3]);
        attrLiquid.setSelected(attrBools[4]);
        attrContact.setSelected(attrBools[5]);
        attrBit1.setSelected(attrBools[6]);
        attrAntiAir.setSelected(attrBools[7]);
        //setup elements area
        boolean[] eleBools = {false, false, false, false, false, false, false, false};
        int eleInt = currentEntryArray[6];
        if (eleInt >= 128)
        {
            eleBools[0] = true;
            eleInt -= 128;
        }
        if (eleInt >= 64)
        {
            eleBools[1] = true;
            eleInt -= 64;
        }
        if (eleInt >= 32)
        {
            eleBools[2] = true;
            eleInt -= 32;
        }
        if (eleInt >= 16)
        {
            eleBools[3] = true;
            eleInt -= 16;
        }
        if (eleInt >= 8)
        {
            eleBools[4] = true;
            eleInt -= 8;
        }
        if (eleInt >= 4)
        {
            eleBools[5] = true;
            eleInt -= 4;
        }
        if (eleInt >= 2)
        {
            eleBools[6] = true;
            eleInt -= 2;
        }
        if (eleInt >= 1)
        {
            eleBools[7] = true;
            eleInt -= 1;
        }
        eleStatus.setSelected(eleBools[7]);
        eleForce.setSelected(eleBools[6]);
        eleElectric.setSelected(eleBools[5]);
        eleCold.setSelected(eleBools[4]);
        eleHeat.setSelected(eleBools[3]);
        eleBlunt.setSelected(eleBools[2]);
        elePierce.setSelected(eleBools[1]);
        eleSlash.setSelected(eleBools[0]);
    }

    /**
     * Private helper method for calculating values of eight
     * radio buttons that represent bit toggles equal to one 
     * byte.  Returns an int that represents an eight bit byte.
     * @param JToggleButton bit0
     * @param JToggleButton bit1
     * @param JToggleButton bit2
     * @param JToggleButton bit3
     * @param JToggleButton bit4
     * @param JToggleButton bit5
     * @param JToggleButton bit6
     * @param JToggleButton bit7
     * @return  int tempByte
     */
    private int eightBitToggles(JToggleButton bit0, JToggleButton bit1, JToggleButton bit2, JToggleButton bit3, JToggleButton bit4, JToggleButton bit5, JToggleButton bit6, JToggleButton bit7)
    {
        int tempByte = 0;
        if (bit7.isSelected())
        {
            tempByte += 128;
        }
        if (bit6.isSelected())
        {
            tempByte += 64;
        }
        if (bit5.isSelected())
        {
            tempByte += 32;
        }
        if (bit4.isSelected())
        {
            tempByte += 16;
        }
        if (bit3.isSelected())
        {
            tempByte += 8;
        }
        if (bit2.isSelected())
        {
            tempByte += 4;
        }
        if (bit1.isSelected())
        {
            tempByte += 2;
        }
        if (bit0.isSelected())
        {
            tempByte += 1;
        }
        return tempByte;
    }

}
